
/* *************************************************************************
 *  File : MTvodo.c
 * -------------------------------------------------------------------------
 *  All rights are reserved by Bo-kyun Na, IntellActive Research Inc.
 * -------------------------------------------------------------------------
 * This example reads an image from /dev/video, transmits compressed video 
 * over the local network, and optionally displays the image in Xwindow.
 * ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"

int count = 0;
struct pollfd pollfds[2];	/* polling file descriptors for socket */

extern int verbose;


/* *************************************************************************
 * void *Thread_TxVideo (void *arg)
 *
 *
 * ************************************************************************* */
void main (int argc, char **argv)
{
  FILE *fp;
  char fname[80];
  int first = 0, userno = 0;
  int i;
  int readable = 0, writable = 0;

  Parameters *param;
  TransceiveState *tstate;

  int vid;
  int CISsize;

  XWA xwa;
  V4L2 v4l2;
  char	devname[64];
  VIMAGE vimage;

  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 
  char outbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  unsigned char *y, *u, *v;


  /* Put in the device node name */
  strcpy(devname, VDO_DEVICE);

  /* Initialization of video devices */
  param = (Parameters *) init_TxVidParam (argc, argv);

  /* Process command line arguments */
  for (i = 1; i < argc; i++) {
    if (!strcmp (argv[i], "-C")) {
      //      switch (argv[++i]) {
      //      case "H261":
      //	docis = H261;
      //	cis_type = "H261";
      //	break;
      //      case "Jpeg" :
      //	docis = JPEG;
      //	cis_type = "Jpeg";
      //	break;
      //      case "Mpeg1":
      //	docis = MPEG;
      //	cis_type = "Mpeg1";
      //	break;
      //      default:
      //      }
    }
    if(!strcmp(argv[i], "-D")) {
      strcpy (devname, argv[++i]);
    }
    if (!strcmp(argv[i], "-i")) {
      strcpy (fname, argv[++i]);
      readable = 1;
      fp = fopen(fname, "r");
      if (fp == NULL) printf ("Can't open image file: %s\n", fname);
      else  printf ("Image file name is %s\n", fname);
    }
    if (!strcmp(argv[i], "-o")) {
      strcpy (fname, argv[++i]);
      writable = 1;
      fp = fopen(fname, "w");
      if (fp == NULL) printf ("Can't open image file: %s\n", fname);
      else  printf ("Image file name is %s\n", fname);
    }
    if(!strcmp(argv[i], "-r")) {
      //      rate = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-s")) {
      //      width = atoi(argv[++i]);
      //      height = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-x")) {
      //    exit_on_done = 1;
    }
  }

  xwa.xdepth = 0;
  /* Pointers for the conversion of RGBtoYUV */
  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, VIDEO);

  /* Function: Init. of video capture to get the video from my camera */
  vid = BV_init_vidcap (devname, &v4l2);

  v4l2.req.count = 2;		/* Num of requested window displays */

  /* Init. and format of xwindow display */
  BV_init_display (&xwa, &v4l2, argc, argv, &vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_encoder ();		/* Initilization of H261 encoder */
  init_decoder ();		/* Initilization of H261 decoder */


  /* Loop processing related to a video frame */
  for (;;) {
    //    sem_wait (&VS);
    //    fprintf (stderr, "SendVidThread ");

    if (readable)
      fread (vimage.data, v4l2.fmt.sizeimage, 1, fp);
    else
      BV_get_vdoframe (vid, vimage.data, &v4l2);

    BV_display_vdoframe (&xwa, &vimage, &v4l2);

    /* Conversion image pixel format */
    //    RGB16toYUV(vimage.data, y, u, v, VDO_WIDTH, VDO_HEIGHT);

    /* Implemenation of H261 encoder */
    //    CISsize = p64EncodeIntraFrame (inbuf, outbuf);
    /*	CISsize = p64EncodeFrame (inbuf, outbuf); */

    //      CISsize = p64EncodeIntraFrame (inbuf, outbuf);

    if (verbose) {
      printf ("raw image = %d bytes\n", v4l2.fmt.sizeimage);
      printf ("Tx:encoded message size = %d\n", CISsize);
    }

    /* Store compressed video data as a file */
    if (writable)	
      fwrite(vimage.data, 1, CISsize, fp);

    /* Packetization and sending of video data */
    if (param->transmit) {
      /* transmit compressed image sequence data */
      //      BV_packetize_vdoframe (outbuf, CISsize, VDO_WIDTH, VDO_HEIGHT, tstate, param);

      /* transmit raw video data */
      BV_packetize_vdoframe (vimage.data, v4l2.fmt.sizeimage, VDO_WIDTH, VDO_HEIGHT, tstate, param);

    }

    //    decode_frame (outbuf, CISsize, vimage.data, first, userno);
    //    BV_display_vdoframe (&xwa, &vimage, &v4l2);

    //    (first++) %= 10;  
    //    thrd_stop (&AR); 
  }
}
