

#include "vdo.h"

extern int once;
extern int verbose;

/* *************************************************************************
 * int BV_close_vid ()
 *
 *
 * ************************************************************************* */
int BV_close_vid (int fd)
{
  int rt;

  rt = close (fd);
  if (rt == -1) {
    perror ("Error closing /dev/video");
    exit (1);
  }

  return rt;
}

/* *************************************************************************
 * void printctrl (struct v4l2_queryctrl *qc)
 *
 *
 * ************************************************************************* */
void printctrl (struct v4l2_queryctrl *qc)
{	
  printf("  %-12s %-6s %8d %8d %7d %8d  %s\n", qc->name,
	 (qc->type == V4L2_CTRL_TYPE_MENU) ? "(menu)" : 
	 (qc->type == V4L2_CTRL_TYPE_BOOLEAN) ? "(bool)" : "(int)",
	 qc->minimum, qc->maximum, qc->step, qc->default_value, qc->catname);
}

/* *************************************************************************
 * int BV_init_vidcap (char *my_device, V4L2 *v4l2)
 *
 *
 * ************************************************************************* */
int BV_init_vidcap (char *my_device, V4L2 *v4l2)
{
  int i;
  int input;
  int err;
  int vid;
  struct v4l2_queryctrl qc;
  struct v4l2_captureparm parm;


  /* Initialization of video device */
  vid = open(my_device, O_RDONLY);
  if (vid < 0) {
    printf("No video device \"%s\"\n", my_device);
    exit (0);
  }

  /* print info of video device */
  if (ioctl(vid, VIDIOC_QUERYCAP, &v4l2->cap)) {
    printf("QUERYCAP returned error %d\n", errno);
    exit (0);
  }

    printf("Device node:  %s\n", my_device);
    printf("Device:       %s\n", v4l2->cap.name);
    printf("Capabilities:");
    if (v4l2->cap.flags & V4L2_FLAG_READ)       printf(" capture via read();");
    if (v4l2->cap.flags & V4L2_FLAG_STREAMING)  printf(" streaming capture;");
    if (v4l2->cap.flags & V4L2_FLAG_PREVIEW)    printf(" automatic preview;");
    if (v4l2->cap.flags & V4L2_FLAG_SELECT)     printf(" select() call;");
    if (v4l2->cap.flags & V4L2_FLAG_TUNER)      printf(" tuner;");
    if (v4l2->cap.flags & V4L2_FLAG_MONOCHROME) printf(" monochrome only;");
    printf("\n");
    if (v4l2->cap.type != V4L2_TYPE_CAPTURE) {
      printf("Not a capture device.\n");
      exit (0);
    }

    printf("Video inputs on this device:\n");
    for (i = 0, err = 0; err == 0; ++i) {	
      struct v4l2_input inp;
      inp.index = i;
      err = ioctl(vid, VIDIOC_ENUMINPUT, &inp);
      if (!err)
	printf("  %d: \"%s\" is a%s, %s audio\n", i, inp.name,
	       (inp.type == V4L2_INPUT_TYPE_TUNER) ?
	       " TV tuner" : "n analog input",
	       (inp.capability & V4L2_INPUT_CAP_AUDIO) ? "has" : "no");
    }

  input = VDO_INPUT;
  if (ioctl(vid, VIDIOC_S_INPUT, input))
    printf("S_INPUT returned error %d\n",errno);

    printf("Video standards supported:");
    for (i = 0, err = 0; err == 0; ++i) {
      struct v4l2_enumstd	estd;
      estd.index = i;
      err = ioctl(vid, VIDIOC_ENUMSTD, &estd);
      if (!err) {
	printf("%c %s", i?',':' ', estd.std.name);
	if (strcmp(VDO_STD, estd.std.name) == 0)
	  v4l2->std = estd.std;
      }
    }
    printf("\n");

    printf("Video capture image formats supported:\n");
    for (i = 0, err = 0; err == 0; ++i) {
      struct v4l2_fmtdesc	fmtd;
      fmtd.index = i;
      err = ioctl(vid, VIDIOC_ENUM_CAPFMT, &fmtd);
      if (!err)
	printf("  %d: %s  (%s)\n", i, fmtd.description,
	       (fmtd.flags & V4L2_FMT_FLAG_COMPRESSED) ?
	       "compressed" : "uncompressed");
    }
    
    printf("Controls supported:\n");
    printf("  Label        type    Minimum  Maximum    Step  Default"
	   " Category\n");
    qc.id = V4L2_CID_BRIGHTNESS;
    if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
      printctrl(&qc);
    qc.id = V4L2_CID_CONTRAST;
    if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
      printctrl(&qc);
    qc.id = V4L2_CID_SATURATION;
    if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
      printctrl(&qc);
    qc.id = V4L2_CID_HUE;
    if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
      printctrl(&qc);

    if (!(v4l2->cap.flags & V4L2_FLAG_READ)) {
      printf("Device does not support the read() call.\n");
      exit (1);
    }

  
  ioctl(vid, VIDIOC_G_PARM, &parm);
  if (ioctl(vid, VIDIOC_S_PARM, &parm))
    printf("S_PARM returned error %d\n",errno);

  if (ioctl(vid, VIDIOC_S_STD, &v4l2->std))
    printf("S_STD returned error %d\n",errno);

  ioctl(vid, VIDIOC_G_FMT, &v4l2->fmt);

  if (ioctl(vid, VIDIOC_S_FMT, &v4l2->fmt)) {
    printf("S_FMT returned error %d\n", errno);
    exit (1);
  }

  return vid;
}


/* *************************************************************************
 * BV_get_vdoframe ()
 *
 *
 * ************************************************************************* */
int BV_get_vdoframe (int vid, char *outbuf, V4L2 *v4l2)
{
  int frame_size;

  if (verbose) {
    printf("Captured Video : %dx%dx%d \"%4.4s\" images\n", v4l2->fmt.width, 
	   v4l2->fmt.height, v4l2->fmt.depth, (char *)&v4l2->fmt.pixelformat);
    printf("                 Each frame has %d bytes\n", v4l2->fmt.sizeimage/8);
  }
  
  frame_size = read(vid, outbuf, v4l2->fmt.sizeimage);

  return frame_size;
}


/* *************************************************************************
 * RGB2YUV ()
 *
 *
 * ************************************************************************* */
void RGB16toYUV(unsigned char *input, unsigned char *y, unsigned char *u, 
		unsigned char *v, int width, int height)
{
  register int i, j;
  register unsigned char *in;
  register short cru, crv, lum;
  register unsigned char *y1 = y, *u1 = u, *v1 = v; 
  const int QCIFwidth = 176;
  const int QCIFheigth = 144;
  static int first = 1;
  static float  mult299[1024], mult587[1024], mult114[1024];
  static float  mult16874[1024], mult33126[1024], mult5[1024];
  static float mult41869[1024], mult08131[1024];
  register unsigned char r, g, b;

  in = input + width * (height - 1) * 2;

  if (first) {
    register int index;
    register int maxValue;

    maxValue = 256;

    for (index = 0; index <= maxValue; index++) {
      mult299[index] = index * 0.29900;
      mult587[index] = index * 0.58700;
      mult114[index] = index * 0.11400;
      mult16874[index] = -0.16874 * index;
      mult33126[index] = -0.33126 * index;
      mult5[index] = index * 0.50000;
      mult41869[index] = -0.41869 * index;
      mult08131[index] = -0.08131 * index;
    }
 
    first = 0;
  }

  for(i = 0; i < QCIFheigth; i++) {
    for(j = 0; j < QCIFwidth; j++) {
      b = (*(in) & 31) << 3;
      g = ((*(in) & 224) >> 2) | (*(in + 1) & 3) << 6;
      r = (*(in + 1) & 124) << 1;

      lum =  mult114[b] + mult587[g] + mult299[r];
   
      *(y1++) = (219 * lum / 255) + 16;

      if(((i % 2) == 1) && ((j % 2) == 1)) {
	cru =  mult5[b] + mult33126[g] + mult16874[r];

	*(u1++) = (224 * cru / 255) + 128;

	crv =  mult08131[b] + mult41869[g] + mult5[r];

	*(v1++) = (224 * crv / 255) + 128;
      }

      in += 2;
    }
    in -= 2 * (width + QCIFwidth);
  }
}

