
#include "vdo.h"

#define X_DEPTH  32


/* *************************************************************************
 * __u32 frame_buffer_format (Display *d, Window w)
 *
 *
 * ************************************************************************* */
__u32 frame_buffer_format (Display *d, Window w, int *x_depth)
{
  __u32	fbf = V4L2_PIX_FMT_RGB565;

  if (*x_depth == 0) {
#ifdef X_DEPTH
    *x_depth = X_DEPTH;
#else
    XWindowAttributes wa;
    XGetWindowAttributes(d, w, &wa);
    x_depth = &wa.depth;
#endif
  }
  if (*x_depth == 15)      fbf = V4L2_PIX_FMT_RGB555;
  else if (*x_depth == 16) fbf = V4L2_PIX_FMT_RGB565;
  else if (*x_depth == 24) fbf = V4L2_PIX_FMT_BGR24;
  else if (*x_depth == 32) fbf = V4L2_PIX_FMT_BGR32;
  else {
    printf("Unrecognized display depth. You may get an X Windows error.\n");
  }
  return fbf;
}

/* *************************************************************************
 * void quit_capture (Widget w, XtPointer client, XtPointer call)
 *
 *
 * ************************************************************************* */
void quit_capture (int vid, Widget w, XtPointer client, XtPointer call)
{
  struct v4l2_performance	perf;
  
  if (vid >= 0) {
    if (ioctl(vid, VIDIOC_G_PERF, &perf) != 0)
      printf("G_PERF returned error code %d\n",errno);
    else
      printf("Captured %d frames\n", perf.frames);
  }
  close (vid);
  exit(0);
}


/* *************************************************************************
 * int BV_init_display ()
 *
 *
 * ************************************************************************* */
void BV_init_display (XWA *xwa, V4L2 *v4l2, int argc, char **argv, VIMAGE *vimage) 
{
  int i;

  Widget toplevel;
  Widget topbox;
  Widget menubox;
  Widget quit_widget;

  /* Set the size of display */
  v4l2->fmt.width  = VDO_WIDTH;
  v4l2->fmt.height = VDO_HEIGHT;

  toplevel = XtAppInitialize(&xwa->app, "IARI", NULL, 0, &argc, argv, NULL, NULL, 0);
  topbox = XtVaCreateManagedWidget("topbox", boxWidgetClass, toplevel,
			    XtNorientation, XtorientVertical, XtNborderWidth,
			    0, XtNhSpace, 0, XtNvSpace, 0, NULL);
  menubox = XtVaCreateManagedWidget("menubox", boxWidgetClass, topbox,
			    XtNorientation, XtorientHorizontal, XtNborderWidth,
			    0, XtNhSpace, 4, XtNvSpace, 1, NULL);
  quit_widget = XtVaCreateManagedWidget("Close!", commandWidgetClass, menubox,
			    XtNborderWidth, 0, NULL);
  XtAddCallback(quit_widget, XtNcallback, (XtCallbackProc) exit, NULL);
  xwa->viewport = 
    XtVaCreateManagedWidget("port", portholeWidgetClass, topbox, XtNwidth,
			   v4l2->fmt.width, XtNheight, v4l2->fmt.height, NULL);
  XtRealizeWidget(toplevel);
  /* Get display pixel format */
  v4l2->fmt.pixelformat = 
    frame_buffer_format(XtDisplay(xwa->viewport),
			DefaultRootWindow(XtDisplay(xwa->viewport)), 
			&(xwa->xdepth));

  xwa->gc = XCreateGC(XtDisplay(xwa->viewport), XtWindow(xwa->viewport), 
		      0, NULL);
  if (xwa->gc == NULL) {
    printf("XCreateGC failed.\n");
    exit (1);
  }

  v4l2->fmt.sizeimage = VDO_WIDTH * VDO_HEIGHT * (xwa->xdepth / 8);

  for (i = 0; i < v4l2->req.count; ++i) {
    vimage[i].data = calloc(1, v4l2->fmt.sizeimage);
    if (vimage[i].data == NULL) {
      printf("malloc(%d) failed\n", v4l2->fmt.sizeimage);
      exit (1);
    }

    vimage[i].ximage = XCreateImage (XtDisplay(xwa->viewport), 
				    DefaultVisual(XtDisplay(xwa->viewport), 0),
				    (xwa->xdepth == 32) ? 24 : xwa->xdepth,
				   ZPixmap, 0, vimage[i].data, v4l2->fmt.width,
				    v4l2->fmt.height, 16, 0);

    if (vimage[i].ximage == NULL) {
      printf("No XImage\n");
      exit (1);
    }
  }
}


/* *************************************************************************
 * int BV_display_vdoframe ()
 *
 *
 * ************************************************************************* */
void BV_display_vdoframe (XWA *xwa, VIMAGE *vimage, V4L2 *v4l2)
{
  XEvent event;

  if (XtIsRealized(xwa->viewport)) 
    XPutImage (XtDisplay(xwa->viewport), XtWindow(xwa->viewport), xwa->gc,
	       vimage->ximage, 0, 0, 0, 0, v4l2->fmt.width, v4l2->fmt.height); 

  else 
    XtAppPeekEvent(xwa->app, &event);

  while (XtAppPending(xwa->app)) {
    XtAppNextEvent(xwa->app, &event);
    XtDispatchEvent(&event);
  }

}


#ifdef CODEC
/* *************************************************************************
 * char *BV_vdo_code (int codec_type, char *input)
 *
 *
 * ************************************************************************* */
char *BV_vdo_code (int codec_type, char *input)
{
  static int first = 1;
  int size;
  char buf[1000];

  switch (codec_type) {
  case H261:
    if (first) {
      size = H261codeIntraFrame (input, buf);
      first = 0;
    } 
    else 
      size = H261codeFrame (&input, buf);
    return buf;

    break;
  case H263: {
    if (first) {
      size = H263codeIntraFrame (input, buf);
      first = 0;
    } 
    else 
      size = H263codeFrame (&input, buf);
    return buf;
  }
  case MPEG:
    break;
  default:
    break;
  }

}


/* *************************************************************************
 * char *BV_vdo_decode (int codec_type, char *input, int size)
 *
 *
 * ************************************************************************* */
char *BV_vdo_decode (int codec_type, char *input, int size)
{
  char *frame;

  switch (codec_type) {
  case H261:
    break;
  case H263: 
    frame = (char *) H263decodeFrame (0, input, size);
    return frame;
  case MPEG:
    break;
  default:
    break;
  }

}

#endif
