
#include "vdo.h"

// #define X_DEPTH 32

extern int verbose;

/* *************************************************************************
 * __u32 frame_buffer_format (Display *d, Window w)
 *
 *
 * ************************************************************************* */
__u32 frame_buffer_format (Display *d, Window w)
{
  __u32	fbf = V4L2_PIX_FMT_RGB565;

  XWindowAttributes wa;
  XGetWindowAttributes(d, w, &wa);

  if (wa.depth == 15) 
    fbf = V4L2_PIX_FMT_RGB555;
  else if (wa.depth == 16)
    fbf = V4L2_PIX_FMT_RGB565;
  else if (wa.depth == 24) 
    fbf = V4L2_PIX_FMT_BGR32;
  else if (wa.depth == 32) 
    fbf = V4L2_PIX_FMT_BGR32;
  else {
    printf("Unrecognized display depth. You may get an X Windows error.\n");
  }
  return fbf;
}

/* *************************************************************************
 * void quit_capture (Widget w, XtPointer client, XtPointer call)
 *
 *
 * ************************************************************************* */
void quit_capture (int vid, Widget w, XtPointer client, XtPointer call)
{
  struct v4l2_performance	perf;
  
  if (vid >= 0) {
    if (ioctl(vid, VIDIOC_G_PERF, &perf) != 0)
      printf("G_PERF returned error code %d\n",errno);
    else
      printf("Captured %d frames\n", perf.frames);
  }
  close (vid);
  exit(0);
}


/* *************************************************************************
 * int BV_init_display ()
 *
 *
 * ************************************************************************* */
char *BV_init_display (XWA *xwa, V4L2 *v4l2, int argc, char **argv, VIMAGE *vimage) 
{
  Widget toplevel;
  Widget topbox;
  Widget menubox;
  Widget quit_widget;


  /* Set the size of display */
  v4l2->fmt.fmt.pix.width  = VDO_WIDTH;
  v4l2->fmt.fmt.pix.height = VDO_HEIGHT;

  toplevel = XtAppInitialize(&xwa->app, "IARI", NULL, 0, &argc, argv, NULL, NULL, 0);
  topbox = XtVaCreateManagedWidget("topbox", boxWidgetClass, toplevel,
			    XtNorientation, XtorientVertical, XtNborderWidth,
			    0, XtNhSpace, 0, XtNvSpace, 0, NULL);
  menubox = XtVaCreateManagedWidget("menubox", boxWidgetClass, topbox,
			    XtNorientation, XtorientHorizontal, XtNborderWidth,
			    0, XtNhSpace, 4, XtNvSpace, 1, NULL);
  quit_widget = XtVaCreateManagedWidget("Close!", commandWidgetClass, menubox,
			    XtNborderWidth, 0, NULL);
  XtAddCallback(quit_widget, XtNcallback, (XtCallbackProc) exit, NULL);
  xwa->viewport = 
    XtVaCreateManagedWidget("port", portholeWidgetClass, topbox, XtNwidth,
			   v4l2->fmt.fmt.pix.width, XtNheight, 
			    v4l2->fmt.fmt.pix.height, NULL);
  XtRealizeWidget(toplevel);

  xwa->gc = XCreateGC(XtDisplay(xwa->viewport),XtWindow(xwa->viewport),0,NULL);
  if (xwa->gc == NULL) {
    printf("XCreateGC failed.\n");
    exit (1);
  }

  /* Get display pixel format from the info of the graphic video card  */
  v4l2->fmt.fmt.pix.pixelformat = frame_buffer_format(XtDisplay(xwa->viewport),
		                  DefaultRootWindow(XtDisplay(xwa->viewport)));

  if (verbose)
    printf ("Display pixel format is %4.4s and the depth is %d\n",
	    (char *)&v4l2->fmt.fmt.pix.pixelformat, xwa->xdepth);
  

  v4l2->fmt.fmt.pix.sizeimage = VDO_WIDTH * VDO_HEIGHT * (xwa->xdepth / 8);

  vimage->data = (char *) calloc (1, v4l2->fmt.fmt.pix.sizeimage);
  if (vimage->data == NULL) {
    printf("malloc(%d) failed\n", v4l2->fmt.fmt.pix.sizeimage);
    exit (1);
  }

  vimage->ximage = XCreateImage (XtDisplay (xwa->viewport), 
				 DefaultVisual (XtDisplay (xwa->viewport), 0),
				 (xwa->xdepth == 32) ? 24 : xwa->xdepth,
				 ZPixmap, 0, vimage->data, 
				 v4l2->fmt.fmt.pix.width,
				 v4l2->fmt.fmt.pix.height,
				 16, 0);

  if (vimage->ximage == NULL) {
    printf("No XImage\n");
    exit (1);
  }

  return vimage->data;
}


/* *************************************************************************
 * int BV_display_vdoframe ()
 *
 *
 * ************************************************************************* */
void BV_display_vdoframe (XWA *xwa, VIMAGE *vimage, V4L2 *v4l2)
{
  XEvent event;

  if (XtIsRealized(xwa->viewport)) 
    XPutImage (XtDisplay(xwa->viewport), XtWindow(xwa->viewport), xwa->gc,
	       vimage->ximage, 0, 0, 0, 0, 
	       v4l2->fmt.fmt.pix.width, v4l2->fmt.fmt.pix.height); 

  else 
    XtAppPeekEvent(xwa->app, &event);

  while (XtAppPending(xwa->app)) {
    XtAppNextEvent(xwa->app, &event);
    XtDispatchEvent(&event);
  }

}

