/* =========================================================================
 * FILE       : libodo.c
 * -----------:-------------------------------------------------------------
 * PROGRAMMER : Bo-kyun Na (bkna@npac.syr.edu), IntellActive Research Inc.
 *            : 1999.10.9.
 * -----------:-------------------------------------------------------------
 * ********** :     All rights are reserved by Bo-kyun Na.        **********
 * -----------:-------------------------------------------------------------
 * Revision   : 1999.10.12. : To libodo.a from audio recordplayer.c
 * -----------:-------------------------------------------------------------
 * This example records audio from /dev/dsp device, transmits compressed audio 
 *    over the local network, and plays the audio on Xwindow.
 * Records several seconds of sound, then echoes it back.  Runs until 
 *    Control-C is pressed. 
 * ========================================================================= */

#include "odo.h"

extern int errno;


/* *************************************************************************
 * Function    : main ()
 * ------------:------------------------------------------------------------
 * Inputs      : none
 * Outputs     : int 
 * Description : void *Thread_TxAudio (void *arg)
 * ************************************************************************* */
int odo_test_main()
{
  int fd;			/* sound device file descriptor */
  int status;			/* return status of system calls */
  int rd, wr;			/* size in byte unit */
  int odo_frame;

  char dev_odo[64];		/* device driver of audio device */
  unsigned char *buf;		/* this buffer holds the digitized audio */

  /* audio device driver */
  strcpy (dev_odo, ODO_DEVICE);		

  odo_frame = ODO_LENGTH * ODO_RATE * ODO_SIZE * ODO_CHANNELS / 8;
  buf = (unsigned char *) malloc(odo_frame);
  if (buf == NULL) {
    printf ("malloc fail: sizeof (buf) = %d\n", odo_frame);
    exit (1);
  }

  /* Initialization of audio */
  fd = BV_ODO_initialize (dev_odo);

  for ( ; ; ) {			/* loop until Control-C */
    printf("Speak to the Microphone:");
    rd = BV_ODO_record (fd, buf, odo_frame);
    printf ("Record size is %d\n", rd);
    //    sleep (1);

    printf("You said:");
    wr = BV_ODO_play (fd, buf, odo_frame);
    printf ("Play size is %d\n", wr);
    //    sleep (1);

    /* wait for playback to complete before recording again */
    status = ioctl(fd, SOUND_PCM_SYNC, 0); 
    if (status == -1)
      perror("SOUND_PCM_SYNC ioctl failed");
  }
}



/* *************************************************************************
 * Function    :  
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
int BV_ODO_initialize (char *dev_odo)
{
  int status;
  int fd;
  int arg;	/* argument for ioctl calls */

  /* open sound device */
  //  fd = open(dev_odo, O_RDWR);
  fd = open(dev_odo, O_WRONLY);
  if (fd <= 0) {
    perror("open of audio device, \"/dev/dsp\", failed");
    exit(1);
  }

  /* set sampling parameters */
  arg = ODO_SIZE;	   /* sample size */
  status = ioctl(fd, SOUND_PCM_WRITE_BITS, &arg);
  if (status == -1)
    perror("SOUND_PCM_WRITE_BITS ioctl failed");
  if (arg != ODO_SIZE)
    perror("unable to set sample size");

  arg = ODO_CHANNELS;  /* mono or stereo */
  status = ioctl(fd, SOUND_PCM_WRITE_CHANNELS, &arg);
  if (status == -1)
    perror("SOUND_PCM_WRITE_CHANNELS ioctl failed");
  if (arg != ODO_CHANNELS)
    perror("unable to set number of channels");

  arg = ODO_RATE;	   /* sampling rate */
  status = ioctl(fd, SOUND_PCM_WRITE_RATE, &arg);
  if (status == -1)
    perror("SOUND_PCM_WRITE_WRITE ioctl failed");

  return fd;
}


/* *************************************************************************
 * Function    : BV_ODO_record ()
 * ------------:------------------------------------------------------------
 * Inputs      : int fd, unsigned char *buf
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
int BV_ODO_record (int fd, unsigned char *buf, int odo_frame)
{
  int status;
  
  status = read(fd, buf, odo_frame); /* record some sound */
  if (status != odo_frame) {
    printf ("fd = %d, status = %d, and errno = %d\n", fd,status, errno);
    perror("read wrong number of bytes");
  }

  return status;
}



/* *************************************************************************
 * Function    : BV_ODO_play ()
 * ------------:------------------------------------------------------------
 * Inputs      : int fd, unsigned char *buf
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
int BV_ODO_play (int fd, unsigned char *buf, int odo_frame)
{
  int status;

  status = write(fd, buf, odo_frame); /* play it back */
  if (status != odo_frame)
    perror("wrote wrong number of bytes");

  return status;
}



/* *************************************************************************
 * Function    :  
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
