/* MTrecv.c	99/03/01  */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>		/* only in linux */
#include <malloc.h>
#include <poll.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>

#include "network.h"
#include "params.h"
#include "transceive.h"


/* *************************************************************************
 * TransceiveState *initialize_receive(Parameters *param, int media_type)
 *
 *
 * ************************************************************************* */
TransceiveState *initialize_receive(Parameters *param, int media_type)
{
  int i;
  int subflag;
  char *tptr;

  int Rxsock;
  struct sockaddr_in *name;
  TransceiveState *receiver;

  unsigned long int inet_addr(const char *cp);
  struct hostent *host, *gethostbyname();


  /* .... create receiver state ....  */
  receiver = (TransceiveState *) calloc(1, sizeof(TransceiveState));
  if (receiver == NULL) return (NULL);

  /* .... open receiver socket and listen to anyone ....   */
  if ((Rxsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("socket");
    free(receiver);
    return (NULL);
  }

  name = receiver->name;
  /*
  name->sin_family = AF_INET;
  name->sin_addr.s_addr = INADDR_ANY;
  name->sin_port = CHANNEL_BASE + param->channel + media_type;

  if (bind(Rxsock, (struct sockaddr *) name, sizeof(*name)) < 0) {
    perror("bind");
    close(Rxsock);
    free(receiver);
    return (NULL);
  }

  receiver->socket = Rxsock;
  */

  for (i = 0; i < param->hostnameknt; i++) {
    tptr = param->hostname[i];
    subflag = 0xffffffff;
    if (tptr[0] == 'S') {
      subflag = 0xffffff00;
      tptr++;
    }

    if (tptr[0] >= '0' && tptr[0] <= '9') {
      name[receiver->nameknt].sin_addr.s_addr = inet_addr(tptr);
      name[receiver->nameknt].sin_addr.s_addr &= subflag;
      name[receiver->nameknt].sin_family = AF_INET;
      name[receiver->nameknt].sin_port = CHANNEL_BASE + param->channel + media_type;
      receiver->nameknt++;
    }
    else {
      if ((host = gethostbyname(tptr)) != NULL) {
	memcpy((char *) &name[receiver->nameknt].sin_addr, 
	       (char *) host->h_addr, host->h_length);
	name[receiver->nameknt].sin_addr.s_addr &=  subflag;
	name[receiver->nameknt].sin_family = AF_INET;
	name[receiver->nameknt].sin_port = CHANNEL_BASE + param->channel + media_type;
	receiver->nameknt++;
      }
      else {
	fprintf(stderr, "Receive gethostbyname '%s'  ", tptr);
	perror("");
      }
    }
  }

  if (verbose) {
    char *tname;
    int i;
    
    for (i = 0; i < receiver->nameknt; i++) {
      tname = "SUBNET";
      if (receiver->name[i].sin_addr.s_addr & 0xff) {
	if ((host = gethostbyaddr ((char *) &receiver->name[i].sin_addr, 4, 
				  AF_INET)) == NULL) {
	  perror("gethostbyaddr");
	  tname = "CAN NOT FIND";
	}
	else {
	  tname = host->h_name;
	}
      }
      
      fprintf(stderr, "Receiving from %d.%d.%d.%d\t\t%s\n",
	      (receiver->name[i].sin_addr.s_addr >> 24) & 0xff,
	      (receiver->name[i].sin_addr.s_addr >> 16) & 0xff,
	      (receiver->name[i].sin_addr.s_addr >> 8) & 0xff,
	      (receiver->name[i].sin_addr.s_addr >> 0) & 0xff, tname);
    }
  }


  if (bind(Rxsock, (struct sockaddr *) name, sizeof (*name)) < 0) {
    perror("bind");
    close(Rxsock);
    free(receiver);
    return (NULL);
  }

  receiver->socket = Rxsock;

  return (receiver);
}


#ifdef XEVENT
/* *************************************************************************
 * void check_events(Display *display)
 *
 *
 * ************************************************************************* */
void check_events(Display *display)
{
  XEvent event;

  if (XPending(*display) > 0) {
    XNextEvent(*display, &event);
    if (event.xany.type == ClientMessage &&
	((XClientMessageEvent *) & event)->data.l[0] == closedownAtom) {
       XCloseDisplay(*xdisplay);
      exit(0);
    }
  }
}

#endif

/* *************************************************************************
 * int receive_packet (TransceiveState *receiver, PacketType *packet)
 *
 *
 * ************************************************************************* */
#ifdef XEVENT
int receive_packet(TransceiveState *receiver, PacketType *packet, Widget *viewport)
#else
int receive_packet(TransceiveState *receiver, PacketType *packet)
#endif
{
  int len;
  int size, packetsize;
  int fromlen;

  packetsize = PACKETSIZE;
  fromlen = sizeof (receiver->name[receiver->session]);
  len = recvfrom(receiver->socket, (unsigned char *) packet, packetsize, 0,
		 (struct sockaddr *) &receiver->name[receiver->session], 
		 (int *) &fromlen);
  //  len = read(receiver->socket, (unsigned char *) packet, packetsize); 
  size = sizeof(PacketHeader) + packet->header.size;

  if (verbose) {
    printf("receiving: %d bytes\n", packet->header.size);
    printf ("frame = %d\n", packet->header.frame);    
  }

  return len;
}


/* *************************************************************************
 * void close_receive(TransceiveState *receiver)
 *
 *
 * ************************************************************************* */
void close_receive(TransceiveState *receiver)
{
  close(receiver->socket);
  free(receiver);
}


/* *************************************************************************
 * Function : int packetnum(unsigned short flag)
 *
 *
 * ************************************************************************* */
int packetnum(unsigned short flag)
{
  int count = 0;

  while ((flag & 0x1) == 1) {
    count++;
    flag >>= 1;
  }
  return count;
}


/* *************************************************************************
 * Function : void create_message (MsgType *msg, PacketType *packetlist[])
 *
 * ************************************************************************* */
void create_message (MsgType *msg, PacketType *packetlist[])
{
  int i;
  unsigned char *curloc;
  int totsize = 0;
  int size;


  curloc = msg->data;
  msg->header = packetlist[0]->header;

  for (i = 0; i < MAXPACKETS; i++) {
    if (packetlist[i]) {
      size = packetlist[i]->header.size;
      memcpy (curloc, packetlist[i]->payload, size);
      totsize += size;
      curloc += size;
    }
  }
  msg->header.size = totsize;
}
