/* @(#)params.c	v0.0 99/03/01  */

#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>

#include "params.h"

extern int verbose;


/* *************************************************************************
 * Function    : Parameters *init_Param(int argc, char **argv) 
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
Parameters *init_Param(int argc, char *argv[])
{
  Parameters *param;
  int i;
  char *fileptr = NULL;


  /* .... create parameter block ....    */
  param = (Parameters *) malloc (sizeof(Parameters));
  if (param == NULL) {
    printf ("malloc fail: param\n");
    return (param);
  }

  /* .... load parameter with default values ....   */
  for (i = 0; i < MAXSENDTO; i++)  param->hostname[i][0] = 0;
  param->hostnameknt = 0;
  param->channel = 0;
  param->transmit = 1;

  param->cis_type = "RAW"; // "H261";
  param->codec = RAW; // H261;
  
  param->rdfile[0] = '\0';
  param->wrfile[0] = '\0';

  param->frames = 18000;
  param->width = -1;
  param->height = -1;
  param->depth = 16;

  param->x = -1;
  param->y = -1;
  param->scalex = 2;
  param->scaley = 2;
  param->image_skip = 0;

  strcpy(param->Twinname, "Local Video : ");
  strncat(param->Twinname, param->cis_type, 5);
  strcpy(param->Rwinname, "Remote Video : ");
  strncat(param->Rwinname, param->cis_type, 5);

  /* host names to transceive */
  //  strncpy(param->hostname[param->hostnameknt], "maat.npac.syr.edu", 64);
  //  param->hostnameknt++;

  strncpy(param->hostname[param->hostnameknt], "newhampshire.npac.syr.edu", 64);
  param->hostnameknt++;

  strncpy(param->hostname[param->hostnameknt], "texas.npac.syr.edu", 64);
  param->hostnameknt++;

  if (fileptr) {
    FILE *fp;

    if ((fp = fopen(fileptr, "r")) == NULL) {
      fprintf(stderr, "Error in file open '%s'\n", fileptr);
      free(param);
      return NULL;
    }
    else {
      while (fscanf(fp, "%s", param->hostname[param->hostnameknt]) == 1 &&
	     param->hostnameknt < MAXSENDTO)
	param->hostnameknt++;
    }
  }

  return (param);
}


/* *************************************************************************
 * Parameters *init_RxVidParam(int argc, char *argv[])
 *
 *
 * ************************************************************************* */
Parameters *init_RxVidParam(int argc, char *argv[])
{
  Parameters *param;

  /* .... create parameter block ....   */
  param = (Parameters *) malloc(sizeof(Parameters));
  if (param == NULL) return (param);

  /* .... load parameter with default values ....   */
  param->hostnameknt = 0;
  param->cis_type = "H261";
  param->codec = H261;
  param->x = -1;
  param->y = -1;
  param->width = 176;
  param->height = 144;
  param->frames = 18000;

  param->scalex = 1;
  param->scaley = 1;

  strcpy(param->Rwinname, "Remote Video : ");
  strncat(param->Rwinname, param->cis_type, 5); /* parse command line args  */

  /* hostnames to receive */
  //  strncpy(param->hostname[param->hostnameknt], "maat.npac.syr.edu", 64);
  //  param->hostnameknt++;

  strncpy(param->hostname[param->hostnameknt], "newhampshire.npac.syr.edu", 64);
  param->hostnameknt++;

  strncpy(param->hostname[param->hostnameknt], "texas.npac.syr.edu", 64);
  param->hostnameknt++;

  return (param);
}


/* *************************************************************************
 * void close_RxVidParam(Parameters * pblock)
 *
 *
 * ************************************************************************* */
void close_RxVidParam(Parameters * pblock)
{
  free(pblock);
}


/* *************************************************************************
 * Parameters *init_TxVidParam(int argc, char *argv[])
 *
 *
 * ************************************************************************* */
Parameters *init_TxVidParam(int argc, char *argv[])
{
  Parameters *param;
  int i;
  char *fileptr = NULL;

  /* .... create parameter block ....    */
  param = (Parameters *) malloc(sizeof(Parameters));
  if (param == NULL)  return (param);

  /* .... load parameter with default values ....   */
  for (i = 0; i < MAXSENDTO; i++)  param->hostname[0][0] = 0;
  param->hostnameknt = 0;
  param->transmit = 1;
  param->channel = 0;

  param->cis_type = "H261";
  param->codec = H261;

  param->frames = 18000;
  param->width = -1;
  param->height = -1;
  param->x = -1;
  param->y = -1;
  param->scalex = 2;
  param->scaley = 2;
  param->image_skip = 0;

  strcpy(param->Twinname, "Local Video : ");
  strncat(param->Twinname, param->cis_type, 5);

  /* hostnames to transmit */
  //  strncpy(param->hostname[param->hostnameknt], optarg, 64);
  //  param->hostnameknt++;

  //  strncpy(param->hostname[param->hostnameknt], "maat.npac.syr.edu", 64);
  //  param->hostnameknt++;

  strncpy(param->hostname[param->hostnameknt], "newhampshire.npac.syr.edu", 64);
  param->hostnameknt++;

  strncpy(param->hostname[param->hostnameknt], "texas.npac.syr.edu", 64);
  param->hostnameknt++;


  if (fileptr) {
    FILE *fp;

    if ((fp = fopen(fileptr, "r")) == NULL) {
      fprintf(stderr, "Error in file open '%s'\n", fileptr);
      free(param);
      return NULL;
    }
    else {
      while (fscanf(fp, "%s", param->hostname[param->hostnameknt]) == 1 &&
	     param->hostnameknt < MAXSENDTO)
	param->hostnameknt++;
    }
  }

  return (param);
}


/* *************************************************************************
 * void close_TxVidParam (Parameters * pblock)
 *
 *
 * ************************************************************************* */
void close_TxVidParam (Parameters * pblock)
{
  free(pblock);
}
