/* MTrecv.c	99/03/01  */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>		/* only in linux */
#include <malloc.h>
#include <poll.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>

#include "network.h"
#include "params.h"
#include "transceive.h"


/* *************************************************************************
 * TransceiveState *initialize_receive(Parameters *param, int media_type)
 *
 *
 * ************************************************************************* */
TransceiveState *initialize_receive(Parameters *param, int media_type)
{
  int Rxsock;
  struct sockaddr_in *name;
  TransceiveState *receiver;


  /* .... create receiver state ....  */
  receiver = (TransceiveState *) calloc(1, sizeof(TransceiveState));
  if (receiver == NULL) return (NULL);

  /* .... open receiver socket and listen to anyone ....   */
  if ((Rxsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("socket");
    free(receiver);
    return (NULL);
  }

  name = receiver->name;
  name->sin_family = AF_INET;
  name->sin_addr.s_addr = INADDR_ANY;
  name->sin_port = CHANNEL_BASE + param->channel + media_type;

  if (bind(Rxsock, (struct sockaddr *) name, sizeof (*name)) < 0) {
    perror("bind");
    close(Rxsock);
    free(receiver);
    return (NULL);
  }

  receiver->socket = Rxsock;

  return (receiver);
}


#ifdef XEVENT
/* *************************************************************************
 * void check_events(Display *display)
 *
 *
 * ************************************************************************* */
void check_events(Display *display)
{
  XEvent event;

  if (XPending(*display) > 0) {
    XNextEvent(*display, &event);
    if (event.xany.type == ClientMessage &&
	((XClientMessageEvent *) & event)->data.l[0] == closedownAtom) {
       XCloseDisplay(*xdisplay);
      exit(0);
    }
  }
}

#endif

/* *************************************************************************
 * int receive_packet (TransceiveState *receiver, PacketType *packet)
 *
 *
 * ************************************************************************* */
#ifdef XEVENT
int receive_packet(TransceiveState *receiver, PacketType *packet, Widget *viewport)
#else
int receive_packet(TransceiveState *receiver, PacketType *packet)
#endif
{
  int len;
  int size, packetsize;
  int fromlen;

  packetsize = PACKETSIZE;
  fromlen = sizeof (receiver->name[receiver->session]);
  len = recvfrom(receiver->socket, (unsigned char *) packet, packetsize, 0,
		 (struct sockaddr *) &receiver->name[receiver->session], 
		 (int *) &fromlen);

  //  len = read(receiver->socket, (unsigned char *) packet, packetsize); 
  size = sizeof(PacketHeader) + packet->header.size;

  /*  if (verbose) {
    printf("receiving: %d bytes\n", packet->header.size);
    printf ("frame = %d\n", packet->header.frame);    
  }
  */

  return len;
}


/* *************************************************************************
 * void close_receive(TransceiveState *receiver)
 *
 *
 * ************************************************************************* */
void close_receive(TransceiveState *receiver)
{
  close(receiver->socket);
  free(receiver);
}


/* *************************************************************************
 * Function : int packetnum(unsigned short flag)
 *
 *
 * ************************************************************************* */
int packetnum(unsigned short flag)
{
  int count = 0;

  while ((flag & 0x1) == 1) {
    count++;
    flag >>= 1;
  }
  return count;
}


/* *************************************************************************
 * Function : void create_message (MsgType *msg, PacketType *packetlist[])
 *
 * ************************************************************************* */
void create_message (MsgType *msg, PacketType *packetlist[])
{
  int i;
  unsigned char *curloc;
  int totsize = 0;
  int size;


  curloc = msg->data;
  msg->header = packetlist[0]->header;

  for (i = 0; i < MAXPACKETS; i++) {
    if (packetlist[i]) {
      size = packetlist[i]->header.size;
      memcpy (curloc, packetlist[i]->payload, size);
      totsize += size;
      curloc += size;
    }
  }
  msg->header.size = totsize;
}


/* *************************************************************************
 * Function    : BV_ODO_receive ()
 * ------------:------------------------------------------------------------
 * Inputs      : TransceiveState *RxState, MsgType *message
 * Outputs     : int 
 * Description : 
 * ************************************************************************* */
MsgType *BV_ODO_receive (TransceiveState *RxState, MsgType *message)
{
  int i;
  int packetcnt = 0;
  int currentframe;

  unsigned int currentsession;
  unsigned short myflags;

  PacketType packets[MAXPACKETS];	/* circular buffer of rec'vd packets */
  PacketType *packetlist[MAXPACKETS];	/* packets for this frame */
  PacketType *curpacket;

    
  /* .... receive video ....  */
  currentframe = 0;
  currentsession = 0;

  myflags = 0xffff;
  for (i = 0; i < MAXPACKETS; i++) packetlist[i] = NULL;

  for (;;) {
    curpacket = &packets[packetcnt++];
    if (packetcnt >= MAXPACKETS) packetcnt = 0;

    receive_packet (RxState, curpacket);

    if (currentsession != curpacket->header.session) {
      /* if you join a conference in progress then the current frame and 
       * session will be grabbed from the first received packet received.   */

      for (i = 0; i < MAXPACKETS; i++) 	packetlist[i] = NULL;
      myflags = 0xffff;
      currentframe = curpacket->header.frame;
      currentsession = curpacket->header.session;
    }

    if (curpacket->header.frame < currentframe) {
      /* if this packet is from an earlier frame disregard it   */
      //      if (verbose)
      //	fprintf(stderr, "received stale packet %d, current frame %d\n",
      //		curpacket->header.frame, currentframe);
      continue;
    }

    if (curpacket->header.frame > currentframe) {
      for (i = 0; i < MAXPACKETS; i++)	packetlist[i] = NULL;
      myflags = 0xffff;
      currentframe = curpacket->header.frame;
    }

    /* save the packet for decoding */
    packetlist[packetnum(curpacket->header.flags)] = curpacket;
    myflags &= curpacket->header.flags;

    if (myflags == 0)  {
      create_message(message, packetlist);
      return message;
    }
  }
}


/* *************************************************************************
 * Function    : BV_VDO_receive ()
 * ------------:------------------------------------------------------------
 * Inputs      : TransceiveState *RxState, MsgType *message
 * Outputs     : MsgType *message 
 * Description : 
 * ************************************************************************* */
MsgType *BV_VDO_receive (TransceiveState *RxState, MsgType *message)
{
  int i;
  int packetcnt = 0;
  int currentframe;

  unsigned int currentsession;
  unsigned short myflags;

  PacketType packets[MAXPACKETS];	/* circular buffer of rec'vd packets */
  PacketType *packetlist[MAXPACKETS];	/* packets for this frame */
  PacketType *curpacket;


  /* .... receive video ....      */
  currentframe = 0;
  currentsession = 0;
   
  myflags = 0xffff;
  for (i = 0; i < MAXPACKETS; i++) packetlist[i] = NULL;

  for (;;) {
    curpacket = &packets[packetcnt++];
    packetcnt %= MAXPACKETS; 

    /* Get video packet from remote if available. */
     receive_packet(RxState, curpacket);

     //  if (verbose) printf ("cursession = %d\n", curpacket->header.session);

    if (curpacket->header.frame < currentframe) {
      /* if this packet is from an earlier frame, disregard it. */
      //      if (verbose)
      //	fprintf(stderr, "received stale packet %d, current frame %d\n",
      //		curpacket->header.frame, currentframe);
      //    thrd_stop (&AR); 
      continue;
    }

    if (curpacket->header.frame > currentframe) {
      if (myflags != 0) {
	//	if (verbose)
      //  fprintf (stderr, "dropping %d due to getting packet %d (flags %d)\n",
	//		   currentframe, curpacket->header.frame, myflags);
      }

      for (i = 0; i < MAXPACKETS; i++) 
	packetlist[i] = NULL;
      myflags = 0xffff;
      currentframe = curpacket->header.frame;
    }

    /* Save the packet for decoding */
    packetlist[packetnum(curpacket->header.flags)] = curpacket;
    myflags &= curpacket->header.flags;

    if (myflags == 0) {
      create_message (message, packetlist);
      return message;
    }
  }
}
