/* =========================================================================
 * FILE     : XmDisplay.c
 * ---------:---------------------------------------------------------------
 * By       : Bo-kyun Na (bkna@npac.syr.edu), IntellActive Research Inc.
 *          : 1999.10.4.
 * Revision : 
 * ========================================================================= */

#include "vdo.h"


/* *************************************************************************  
 * Function    : frame_buffer_format()
 * ------------:------------------------------------------------------------
 * Inputs      : Display *d, Window w, XWA *xwa
 * Outputs     :
 * Description :
 * ************************************************************************* */
__u32 frame_buffer_format(Display *d, Window w, XWA *xwa)
{
  __u32	fbf = V4L2_PIX_FMT_RGB565;

  XWindowAttributes wa;
  XGetWindowAttributes(d, w, &wa);

  xwa->xdepth = wa.depth;
  if (wa.depth == 15)
    fbf = V4L2_PIX_FMT_RGB555;
  else if (wa.depth == 16)
    fbf = V4L2_PIX_FMT_RGB565;
  else if (wa.depth == 24)
    fbf = V4L2_PIX_FMT_BGR24;
  else if (wa.depth == 32)
    fbf = V4L2_PIX_FMT_BGR32;
  else {
    printf("Unrecognized display depth. You may get an X Windows error.\n");
  }

  return fbf;
}


/* *************************************************************************  
 * Function    : stop_capturing()
 * ------------:------------------------------------------------------------
 * Inputs      : VIMAGE *vimage
 * Outputs     :
 * Description :
 * ************************************************************************* */
void stop_capturing(VIMAGE *vimage)
{
  if (vimage->vid >= 0)	{
    if (vimage->data) free(vimage->data);
    vimage->data = NULL;
    if (vimage->ximage) vimage->ximage = NULL;
  }
}


/* *************************************************************************  
 * Function    : start_capturing()
 * ------------:------------------------------------------------------------
 * Inputs      : VIMAGE *vimage, XWA *xwa, V4L2 *v4l2
 * Outputs     :
 * Description :
 * ************************************************************************* */
int start_capturing(VIMAGE *vimage, XWA *xwa, V4L2 *v4l2)
{
  int	err;
  Arg	arg[2];

  v4l2->fmt.fmt.pix.pixelformat = xwa->pixelfmt;

  err = ioctl(vimage->vid, VIDIOC_S_FMT, &v4l2->fmt);
  if (err) {
    printf("S_FMT returned error %d\n",errno);
    return 1;
  }

  vimage->data = malloc(v4l2->fmt.fmt.pix.sizeimage);
  if (vimage->data == NULL) {
    printf("malloc(%d) failed\n", v4l2->fmt.fmt.pix.sizeimage);
    return 1;
  }

  vimage->ximage = 
    XCreateImage (XtDisplay(xwa->viewport),
		  DefaultVisual(XtDisplay(xwa->viewport), 0),
		  (xwa->xdepth == 32) ? 24 : xwa->xdepth, ZPixmap, 0, 
		  vimage->data,
		  v4l2->fmt.fmt.pix.width, v4l2->fmt.fmt.pix.height, 16, 0);
  if (vimage->ximage == NULL) {
    printf("No XImage\n");
    return 1;
  }

  XtSetArg(arg[0], XtNwidth, max(160, v4l2->fmt.fmt.pix.width));
  XtSetArg(arg[1], XtNheight, max(120, v4l2->fmt.fmt.pix.height));
  XtSetValues(xwa->viewport, arg, 2);
  XtRealizeWidget(xwa->toplevel);

  return 0;
}


/* *************************************************************************  
 * Function    :
 * ------------:------------------------------------------------------------
 * Inputs      :
 * Outputs     :
 * Description :
 * ************************************************************************* */
void quit_w(Widget w, XtPointer client, XtPointer call, VIMAGE *vimage)
{
  struct v4l2_performance	perf;

  if (vimage->vid >= 0)	{
    if (ioctl(vimage->vid, VIDIOC_G_PERF, &perf) != 0)
      printf("G_PERF returned error code %d\n",errno);
    else
      printf("Captured %d frames\n", perf.frames);

    stop_capturing(vimage);
    close(vimage->vid);
  }
  exit(0);
}


/* *************************************************************************  
 * Function    : std_menu_callback()
 * ------------:------------------------------------------------------------
 * Inputs      : Widget w, XtPointer client, XtPointer call, 
 *  	         VIMAGE *vimage, XWA *xwa, V4L2 *v4l2
 * Outputs     :
 * Description :
 * ************************************************************************* */
void std_menu_callback(Widget w, XtPointer client, XtPointer call, 
		       VIMAGE *vimage, XWA *xwa, V4L2 *v4l2)
{
  StdMenuItem	*smi = (StdMenuItem *)client;

  stop_capturing(vimage);
  ioctl(vimage->vid, VIDIOC_S_STD, &smi->std);
  start_capturing(vimage, xwa, v4l2);
}


/* *************************************************************************  
 * Function    : size_menu_callback()
 * ------------:------------------------------------------------------------
 * Inputs      : Widget w, XtPointer client, XtPointer call,
 *		 VIMAGE *vimage, XWA *xwa, V4L2 *v4l2
 * Outputs     :
 * Description :
 * ************************************************************************* */
void size_menu_callback(Widget w, XtPointer client, XtPointer call,
			VIMAGE *vimage, XWA *xwa, V4L2 *v4l2)
{
  int	i = (int)client;

  v4l2->fmt.fmt.pix.width = xwa->capturesize[i].width;
  v4l2->fmt.fmt.pix.height = xwa->capturesize[i].height;

  stop_capturing(vimage);
  ioctl(vimage->vid, VIDIOC_S_FMT, &v4l2->fmt);
  start_capturing(vimage, xwa, v4l2);
}


/* *************************************************************************
 * Function    : BV_init_vdo_display ()
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
int BV_vdo_init_display (int argc, char **argv, 
			 VIMAGE *vimage, V4L2 *v4l2, XWA *xwa)
{
  int i;
  int err;

  StdMenuItem	stditem[MAXSTDS];

  Widget		w_topbox;
  Widget		w_menubox;
  Widget		w_quit;
  Widget		w_stdbutton;
  Widget		w_stdmenu;
  Widget		w_sizebutton;
  Widget		w_sizemenu;


  ioctl(vimage->vid, VIDIOC_G_FMT, &v4l2->fmt);
  v4l2->fmt.fmt.pix.width = VDO_WIDTH;
  v4l2->fmt.fmt.pix.height = VDO_HEIGHT;
  ioctl(vimage->vid, VIDIOC_S_FMT, &v4l2->fmt);

  xwa->toplevel = XtAppInitialize (&xwa->app, "xvideum",
				NULL, 0, 
				&argc, argv, 
				NULL, NULL, 0);
  w_topbox = XtVaCreateManagedWidget ("topbox",
				     boxWidgetClass, xwa->toplevel,
				     XtNorientation, XtorientVertical,
				     XtNborderWidth, 0,
				     XtNhSpace, 0,
				     XtNvSpace, 0,
				     NULL);
  w_menubox = XtVaCreateManagedWidget ("menubox",
				      boxWidgetClass, w_topbox,
				      XtNorientation, XtorientHorizontal,
				      XtNborderWidth, 0,
				      XtNhSpace, 4,
				      XtNvSpace, 1,
				      NULL);

  w_stdbutton = XtVaCreateManagedWidget ("menubutton",
					menuButtonWidgetClass, w_menubox,
					XtNlabel, "Standard",
					XtNmenuName, "standard",
					XtNborderWidth, 0,
					NULL);
  w_stdmenu = XtVaCreatePopupShell ("standard", /* same as XtNmenuName & static */
				    simpleMenuWidgetClass, w_stdbutton,
				    NULL);
  for (i = 0; ; ++i) {
    struct v4l2_enumstd	estd;
    estd.index = i;
    err = ioctl(vimage->vid, VIDIOC_ENUMSTD, &estd);
    if (err)
      break;
    stditem[i].std = estd.std;
    stditem[i].w_item = XtVaCreateManagedWidget (estd.std.name, 
						smeBSBObjectClass, w_stdmenu,
						 XtNlabel, estd.std.name, NULL);
    XtAddCallback (stditem[i].w_item, XtNcallback, 
		   (XtCallbackProc) std_menu_callback, &stditem[i]);
  }

  w_sizebutton = XtVaCreateManagedWidget ("menubutton",
					  menuButtonWidgetClass, w_menubox,
					  XtNlabel, "Zoom",
					  XtNmenuName, "Zoom",
					  XtNborderWidth, 0, NULL);
  w_sizemenu = XtVaCreatePopupShell ("Zoom", /* same as XtNmenuName & static */
				     simpleMenuWidgetClass, w_sizebutton, NULL);
  for (i = 0; i < sizeof(xwa->capturesize)/sizeof(xwa->capturesize[0]); ++i) {
    char sizestr[32];
    Widget w;

    //    sprintf(sizestr, "%d x %d", 
    //	    xwa->capturesize[i].width, xwa->capturesize[i].height);
    sprintf(sizestr, "%d", xwa->capturesize[i].width);
    w = XtVaCreateManagedWidget (sizestr, smeBSBObjectClass, w_sizemenu,
				 XtNlabel, sizestr, NULL);
    XtAddCallback (w, XtNcallback, (XtCallbackProc) size_menu_callback, (char *)i);
  }

  w_quit = XtVaCreateManagedWidget ("Quit!", commandWidgetClass, w_menubox,
				    XtNborderWidth, 0, NULL);
  XtAddCallback (w_quit, XtNcallback, (XtCallbackProc) quit_w, NULL);
  xwa->viewport = XtVaCreateManagedWidget ("port",
					portholeWidgetClass, w_topbox,
					XtNwidth, VDO_WIDTH,
					XtNheight, VDO_HEIGHT,
					NULL);
  XtRealizeWidget(xwa->toplevel);

  xwa->gc = XCreateGC(XtDisplay(xwa->viewport), XtWindow(xwa->viewport), 0, NULL);
  if (xwa->gc == NULL) {
    printf("XCreateGC failed.\n");
    return 1;
  }

  if (start_capturing(vimage, xwa, v4l2)) {
    return 1;
  }


}


/* *************************************************************************
 * Function    : BV_vdo_display ()
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
 void BV_vdo_display (VIMAGE *vimage, V4L2 *v4l2, XWA *xwa)
{
  XEvent event;

  if (XtIsRealized(xwa->viewport)) {
    read(vimage->vid, vimage->data, v4l2->fmt.fmt.pix.sizeimage);
    XPutImage(XtDisplay(xwa->viewport), XtWindow(xwa->viewport), xwa->gc,
	      vimage->ximage, 0, 0, 0, 0, 
	      v4l2->fmt.fmt.pix.width, v4l2->fmt.fmt.pix.height);

  }
  else {
    XtAppPeekEvent(xwa->app, &event);
  }

  while (XtAppPending(xwa->app)) {
    XtAppNextEvent(xwa->app, &event);
    switch (event.type) {
    default:
      break;
    }
    XtDispatchEvent(&event);
  }
}


/* *************************************************************************
 * Function    : 
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */



/* *************************************************************************
 * Function    : 
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
