/* MTrecv.c	99/03/01  */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>

#include "network.h"
#include "params.h"
#include "transceive.h"

/* To use select (), instead of poll () */
fd_set recvmask;
struct timeval timeout;


TransceiveState *initialize_receive(Parameters *param, int media_type)
{
  TransceiveState *receiver;
  int Rxsock;
  struct sockaddr_in *name;

  /* .... create receiver state ....  */
  receiver = (TransceiveState *) malloc (sizeof (TransceiveState));
  if (receiver == NULL) return (NULL);

  /* .... open receiver socket and listen to anyone ....   */
  if ((Rxsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("socket");
    free(receiver);
    return (NULL);
  }

  name = receiver->name;
  name->sin_family = AF_INET;
  name->sin_addr.s_addr = INADDR_ANY;
  name->sin_port = CHANNEL_BASE + param->channel + media_type;

  if (bind(Rxsock, (struct sockaddr *) name, sizeof(*name)) < 0) {
    perror("bind");
    close(Rxsock);
    free(receiver);
    return (NULL);
  }

  receiver->socket = Rxsock;

  /* instead of poll (), select () works to detect events audio and video */
  FD_ZERO (&recvmask);
  FD_SET (receiver->socket, &recvmask);
  timeout.tv_sec = 0;
  timeout.tv_usec = 10000;

  if (select (3, &recvmask, (fd_set *) 0, (fd_set *) 0, (struct timeval *) &timeout) < 0) {
    perror ("select error");
    exit (0);
  }

  return (receiver);
}


int receive_packet(TransceiveState *receiver, PacketType *packet)
{
  int len;
  int size;
  int fromlen;

  /* Polling by select (): Check if any activity on socket  */
  if (FD_ISSET (receiver->socket, &recvmask)) { 
    fromlen = sizeof (receiver->name[receiver->session]);
    len = recvfrom(receiver->socket, (char *) packet, sizeof(PacketType), 0,
		   (struct sockaddr *) &receiver->name[receiver->session], 
		   (int *) &fromlen);
  }

  size = sizeof(PacketHeader) + packet->header.size;

  if (len == size) return 0;
  return 1;
}


void close_receive(TransceiveState * receiver)
{
  close(receiver->socket);
  free(receiver);
}



