/* MTrecv.c	99/03/01  */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>		/* only in linux */
#include <malloc.h>
#include <poll.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>

#include "network.h"
#include "params.h"
#include "transceive.h"


/* *************************************************************************
 * TransceiveState *initialize_receive(Parameters *param, int media_type)
 *
 *
 * ************************************************************************* */
TransceiveState *initialize_receive(Parameters *param, int media_type)
{
  TransceiveState *receiver;
  int Rxsock;
  struct sockaddr_in *name;

  /* .... create receiver state ....  */
  receiver = (TransceiveState *) calloc(1, sizeof(TransceiveState));
  if (receiver == NULL) return (NULL);

  /* .... open receiver socket and listen to anyone ....   */
  if ((Rxsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("socket");
    free(receiver);
    return (NULL);
  }
  name = receiver->name;
  name->sin_family = AF_INET;
  name->sin_addr.s_addr = INADDR_ANY;
  name->sin_port = CHANNEL_BASE + param->channel + media_type;

  if (bind(Rxsock, (struct sockaddr *) name, sizeof(*name)) < 0) {
    perror("bind");
    close(Rxsock);
    free(receiver);
    return (NULL);
  }

  receiver->socket = Rxsock;

  {
    int value;

    /* UNIX not for Linux */
    //	  ioctl(Rxsock, I_GRDOPT, &value);
    //	  ioctl(Rxsock, I_SRDOPT, RMSGD);

    /* For Linux */
    value = PACKETSIZE;
    ioctl(Rxsock, FIONREAD, value); 
    //    ioctl(Rxsock, _IOC_READ, value); 
    //    ioctl(Rxsock, _IOC_READ);
  }

  /* for poll to tell us if there is a packet waiting for read */
  pollfds[media_type].fd = receiver->socket;;
  pollfds[media_type].events = POLLIN;
  pollfds[media_type].revents = 0;

  return (receiver);
}

#ifdef XEVENT
/* *************************************************************************
 * void check_events(Display *display)
 *
 *
 * ************************************************************************* */
void check_events(Display *display)
{
  XEvent event;

  if (XPending(*display) > 0) {
    XNextEvent(*display, &event);
    if (event.xany.type == ClientMessage &&
	((XClientMessageEvent *) & event)->data.l[0] == closedownAtom) {
       XCloseDisplay(*xdisplay);
      exit(0);
    }
  }
}

#endif

/* *************************************************************************
 * int receive_packet (TransceiveState *receiver, PacketType *packet)
 *
 *
 * ************************************************************************* */
#ifdef XEVENT
int receive_packet(TransceiveState *receiver, PacketType *packet, Widget *viewport)
#else
int receive_packet(TransceiveState *receiver, PacketType *packet)
#endif
{
  int len;
  int size, packetsize;
  int fromlen;

  packetsize = PACKETSIZE;
  fromlen = sizeof (receiver->name[receiver->session]);
  len = recvfrom(receiver->socket, (unsigned char *) packet, packetsize, 0,
		 (struct sockaddr *) &receiver->name[receiver->session], 
		 (int *) &fromlen);
  //  len = read(receiver->socket, (unsigned char *) packet, packetsize); 
  size = sizeof(PacketHeader) + packet->header.size;

  if (verbose) {
    printf("receiving: %d bytes\n", packet->header.size);
    printf ("frame = %d\n", packet->header.frame);    
  }

  return len;
}


/* *************************************************************************
 * void close_receive(TransceiveState *receiver)
 *
 *
 * ************************************************************************* */
void close_receive(TransceiveState *receiver)
{
  close(receiver->socket);
  free(receiver);
}


/* *************************************************************************
 * Function : int packetnum(unsigned short flag)
 *
 *
 * ************************************************************************* */
int packetnum(unsigned short flag)
{
  int count = 0;

  while ((flag & 0x1) == 1) {
    count++;
    flag >>= 1;
  }
  return count;
}


/* *************************************************************************
 * Function : void create_message (MsgType *msg, PacketType *packetlist[])
 *
 * ************************************************************************* */
void create_message (MsgType *msg, PacketType *packetlist[])
{
  int i;
  unsigned char *curloc;
  int totsize = 0;
  int size;


  curloc = msg->data;
  msg->header = packetlist[0]->header;

  for (i = 0; i < MAXPACKETS; i++) {
    if (packetlist[i]) {
      size = packetlist[i]->header.size;
      memcpy (curloc, packetlist[i]->payload, size);
      totsize += size;
      curloc += size;
    }
  }
  msg->header.size = totsize;
}
