

#include "vdo.h"

extern int verbose;


/* *************************************************************************  
 * Function    :
 * Inputs      :
 * Outputs     :
 * Description :
 * 
 *
 * ************************************************************************* */
int BV_close_vid (int fd)
{
  int rt;

  rt = close (fd);
  if (rt == -1) {
    perror ("Error closing /dev/video");
    exit (1);
  }

  return rt;
}

/* *************************************************************************  
 * Function    :
 * Inputs      :
 * Outputs     :
 * Description :
 * 
 *
 * ************************************************************************* */
void stop_capturing()
{
  int	i;

  if (vid >= 0)	{
    i = V4L2_BUF_TYPE_CAPTURE;
    ioctl(vid, VIDIOC_STREAMOFF, &i);
    if (vimage.data)
      munmap(vimage.data, vimage.vidbuf.length);
    vimage.data = NULL;
    if (vimage.ximage)
      vimage.ximage = NULL;
  }
  
}


/* *************************************************************************  
 * Function    :
 * Inputs      :
 * Outputs     :
 * Description :
 * 
 *
 * ************************************************************************* */
int start_capturing()
{
  struct v4l2_requestbuffers req;
  int	err;
  Arg	arg[2];

#ifdef VDO_PIXELFORMAT
  fmt.fmt.pix.pixelformat = VDO_PIXELFORMAT;
#else
  fmt.fmt.pix.pixelformat = frame_buffer_format(XtDisplay(w_viewport),
				      DefaultRootWindow(XtDisplay(w_viewport)));
#endif

  err = ioctl(vid, VIDIOC_S_FMT, &fmt);
  if (err) {
    printf("S_FMT returned error %d\n",errno);
    return 1;
  }

  req.type = V4L2_BUF_TYPE_CAPTURE;
  err = ioctl(vid, VIDIOC_REQBUFS, &req);
  if (err < 0 || req.count < 1)	{
    printf("REQBUFS returned error %d, count %d\n", errno,req.count);
    return 1;
  }
  
  vimage.vidbuf.type = V4L2_BUF_TYPE_CAPTURE;
  err = ioctl(vid, VIDIOC_QUERYBUF, &(vimage.vidbuf));
  if (err < 0) {
    printf("QUERYBUF returned error %d\n",errno);
    return 1;
  }
  vimage.data = mmap(0, vimage.vidbuf.length, PROT_READ, MAP_SHARED, vid, vimage.vidbuf.offset);
  if ((int)vimage.data == -1) {
    printf("mmap() returned error %d\n", errno);
    return 1;
  }
  
  if ((err = ioctl(vid, VIDIOC_QBUF, &(vimage.vidbuf)))) {
    printf("QBUF returned error %d\n",errno);
    return 1;
  }
  err = ioctl(vid, VIDIOC_STREAMON, &(vimage.vidbuf.type));
  if (err)
    printf("STREAMON returned error %d\n",errno);

  vimage.ximage = XCreateImage(XtDisplay(w_viewport),
			       DefaultVisual(XtDisplay(w_viewport), 0),
			       (x_depth == 32) ? 24 : x_depth,
			       ZPixmap, 0, vimage.data,
			       fmt.fmt.pix.width, fmt.fmt.pix.height, 
			       16, 0);
  if (vimage.ximage == NULL) {
    printf("No XImage\n");
    return 1;
  }

  XtSetArg(arg[0], XtNwidth, max(160, fmt.fmt.pix.width));
  XtSetArg(arg[1], XtNheight, max(120, fmt.fmt.pix.height));
  XtSetValues(w_viewport, arg, 2);
  XtRealizeWidget(w_toplevel);

  return 0;
}


/* *************************************************************************  
 * Function    :
 * Inputs      :
 * Outputs     :
 * Description :
 * ************************************************************************* */
void printctrl (struct v4l2_queryctrl *qc)
{	
  printf("  %-12s %-6s %8d %8d %7d %8d  %s\n", qc->name,
	 (qc->type == V4L2_CTRL_TYPE_MENU) ? "(menu)" : 
	 (qc->type == V4L2_CTRL_TYPE_BOOLEAN) ? "(bool)" : "(int)",
	 qc->minimum, qc->maximum, qc->step, qc->default_value, qc->group);
}


/* *************************************************************************  
 * Function    : BV_init_capture_vdoframe ()
 * Inputs      : char *my_device, V4L2 *v4l2
 * Outputs     :
 * Description :
 * ************************************************************************* */
int BV_init_capture_vdoframe (char *my_device, V4L2 *v4l2)
{
  int i;
  int input;
  int err;
  int vid;
  struct v4l2_queryctrl qc;

  /* Initialization of video device */
  vid = open(my_device, O_RDONLY);
  if (vid < 0) {
    printf("No video device \"%s\"\n", my_device);
    return 1;
  }

  err = ioctl(vid, VIDIOC_QUERYCAP, &v4l2->cap);
  if (err) {
    printf("QUERYCAP returned error %d\n", errno);
    return 1;
  }

  if (verbose) {
    printf("Device node:  %s\n", my_device);
    printf("Device:       %s\n", cap.name);
    printf("Capabilities:");
    if (v4l2->cap.flags & V4L2_FLAG_READ)       printf(" capture via read();");
    if (v4l2->cap.flags & V4L2_FLAG_STREAMING)  printf(" streaming capture;");
    if (v4l2->cap.flags & V4L2_FLAG_PREVIEW)    printf(" automatic preview;");
    if (v4l2->cap.flags & V4L2_FLAG_SELECT)     printf(" select() call;");
    if (v4l2->cap.flags & V4L2_FLAG_TUNER)      printf(" tuner;");
    if (v4l2->cap.flags & V4L2_FLAG_MONOCHROME) printf(" monochrome only;");
    printf("\n");
    if (v4l2->cap.type != V4L2_TYPE_CAPTURE) {
      printf("Not a capture device.\n");
      return 1;
    }
  }
  if (verbose)
    printf("Video inputs on this device:\n");
  for (i = 0, err = 0; err == 0; ++i) {	
    struct v4l2_input inp;
    inp.index = i;
    err = ioctl(vid, VIDIOC_ENUMINPUT, &inp);
    if ((!err) && verbose)
      printf("  %d: \"%s\" is a%s, %s audio\n", i, inp.name,
	    (inp.type == V4L2_INPUT_TYPE_TUNER) ? "TV tuner" : "n analog input",
	     (inp.capability & V4L2_INPUT_CAP_AUDIO) ? "has" : "no");
  }

  input = VDO_INPUT;
  err = ioctl(vid, VIDIOC_S_INPUT, &input);
  if (err)
    printf("S_INPUT returned error %d\n",errno);

  if (verbose)
    printf("Video standards supported:");
  for (i = 0, err = 0; err == 0; ++i) {
    struct v4l2_enumstd	estd;
    estd.index = i;
    err = ioctl(vid, VIDIOC_ENUMSTD, &estd);
    if (!err && verbose) {
      printf("%c %s", i?',':' ', estd.std.name);
    }
  }
  if (verbose) 
    printf("\n");

  if (verbose)
    printf("Video capture image formats supported:\n");

  for (i = 0, err = 0; err == 0; ++i) {
    struct v4l2_fmtdesc	fmtd;

    fmtd.index = i;
    err = ioctl(vid, VIDIOC_ENUM_PIXFMT, &fmtd);
    if (!err && verbose)
      printf("  %d: %s  (%s)\n", i, fmtd.description,
	     (fmtd.flags & V4L2_FMT_FLAG_COMPRESSED) ?
	     "compressed" : "uncompressed");
  }

  if (verbose) {
    printf("Controls supported:\n");
    printf("  Label        type    Minimum  Maximum    Step  Default Category\n");
  }
  qc.id = V4L2_CID_BRIGHTNESS;
  if ((ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0) && verbose)
    printctrl(&qc);
  qc.id = V4L2_CID_CONTRAST;
  if ((ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0) && verbose)
    printctrl(&qc);
  qc.id = V4L2_CID_SATURATION;
  if ((ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0) && verbose)
    printctrl(&qc);
  qc.id = V4L2_CID_HUE;
  if ((ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0) && verbose)
    printctrl(&qc);

  if (!(cap.flags & V4L2_FLAG_STREAMING)) {
    printf("Device does not support streaming capture.\n");
    return 0;
  }

  v4l2->parm.type = V4L2_BUF_TYPE_CAPTURE;
  ioctl(vid, VIDIOC_G_PARM, &v4l2->parm);

  /* set proper framerate */
  v4l2->parm.parm.capture.capturemode |= V4L2_CAP_TIMEPERFRAME;
  v4l2->parm.parm.capture.timeperframe = VDO_FRAMERATE;

  err = ioctl(vid, VIDIOC_S_PARM, &v4l2->parm);
  if (err)
    printf("S_PARM returned error %d\n",errno);

  v4l2->fmt.type = V4L2_BUF_TYPE_CAPTURE;
  ioctl(vid, VIDIOC_G_FMT, &v4l2->fmt);
  v4l2->fmt.fmt.pix.width = VDO_WIDTH;
  v4l2->fmt.fmt.pix.height = VDO_HEIGHT;
  ioctl(vid, VIDIOC_S_FMT, &v4l2->fmt);

  if (verbose) {
    printf("Capturing %dx%dx%d \"%4.4s\" images\n", v4l2->fmt.fmt.pix.width, 
	   v4l2->fmt.fmt.pix.height, v4l2->fmt.fmt.pix.depth, 
	   (char *)&v4l2->fmt.fmt.pix.pixelformat);
    printf("===> Frame has %d bytes each\n", v4l2->fmt.fmt.pix.sizeimage);
  }

  return vid;
}



int OLD_BV_init_vidcap (char *my_device, V4L2 *v4l2)
{
  int i;
  int input;
  int err;
  int vid;
  struct v4l2_queryctrl qc;
  struct v4l2_captureparm parm;

  /* Initialization of video device */
  vid = open(my_device, O_RDONLY);
  if (vid < 0) {
    printf("No video device \"%s\"\n", my_device);
    exit (0);
  }

  /* print info of video device */
  if (ioctl(vid, VIDIOC_QUERYCAP, &v4l2->cap)) {
    printf("QUERYCAP returned error %d\n", errno);
    exit (0);
  }

  if (verbose) {
    printf("Device node:  %s\n", my_device);
    printf("Device:       %s\n", v4l2->cap.name);
    printf("Capabilities:");
    if (v4l2->cap.flags & V4L2_FLAG_READ)       printf(" capture via read();");
    if (v4l2->cap.flags & V4L2_FLAG_STREAMING)  printf(" streaming capture;");
    if (v4l2->cap.flags & V4L2_FLAG_PREVIEW)    printf(" automatic preview;");
    if (v4l2->cap.flags & V4L2_FLAG_SELECT)     printf(" select() call;");
    if (v4l2->cap.flags & V4L2_FLAG_TUNER)      printf(" tuner;");
    if (v4l2->cap.flags & V4L2_FLAG_MONOCHROME) printf(" monochrome only;");
    printf("\n");
    if (v4l2->cap.type != V4L2_TYPE_CAPTURE) {
      printf("Not a capture device.\n");
      exit (0);
    }
  }

  if (verbose)
    printf("Video inputs on this device:\n");
  for (i = 0, err = 0; err == 0; ++i) {	
    struct v4l2_input inp;
    inp.index = i;
    err = ioctl(vid, VIDIOC_ENUMINPUT, &inp);
    if ((!err) && verbose)
      printf("  %d: \"%s\" is a%s, %s audio\n", i, inp.name,
	     (inp.type == V4L2_INPUT_TYPE_TUNER) ?
	     " TV tuner" : "n analog input",
	     (inp.capability & V4L2_INPUT_CAP_AUDIO) ? "has" : "no");
  }

  input = VDO_INPUT;
  if (ioctl(vid, VIDIOC_S_INPUT, input))
    printf("S_INPUT returned error %d\n",errno);

  if (verbose)
    printf("Video standards supported:");
  for (i = 0, err = 0; err == 0; ++i) {
    struct v4l2_enumstd	estd;
    estd.index = i;
    err = ioctl(vid, VIDIOC_ENUMSTD, &estd);
    if (!err) {
      if (verbose)
	printf("%c %s", i?',':' ', estd.std.name);
      if (strcmp(VDO_STD, estd.std.name) == 0)
	v4l2->std = estd.std;
    }
  }
  if (verbose)
    printf("\n");

  if (verbose)
    printf("Video capture image formats supported:\n");
  for (i = 0, err = 0; err == 0; ++i) {
    struct v4l2_fmtdesc	fmtd;
    fmtd.index = i;
    err = ioctl(vid, VIDIOC_ENUM_CAPFMT, &fmtd);
    if ((!err) && verbose)
      printf("  %d: %s  (%s)\n", i, fmtd.description,
	     (fmtd.flags & V4L2_FMT_FLAG_COMPRESSED) ?
	     "compressed" : "uncompressed");
  }

  if (verbose) {
  printf("Controls supported:\n");
  printf("  Label        type    Minimum  Maximum    Step  Default"
	 " Category\n");
  }
  qc.id = V4L2_CID_BRIGHTNESS;
  if ((ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0) && verbose)  printctrl(&qc);
  qc.id = V4L2_CID_CONTRAST;
  if ((ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0) && verbose)  printctrl(&qc);
  qc.id = V4L2_CID_SATURATION;
  if ((ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0) && verbose)  printctrl(&qc);
  qc.id = V4L2_CID_HUE;
  if ((ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0) && verbose)  printctrl(&qc);

  if (!(v4l2->cap.flags & V4L2_FLAG_READ)) {
    printf("Device does not support the read() call.\n");
    exit (1);
  }

  ioctl(vid, VIDIOC_G_PARM, &parm);
  if (ioctl(vid, VIDIOC_S_PARM, &parm))
    printf("S_PARM returned error %d\n",errno);

  if (ioctl(vid, VIDIOC_S_STD, &v4l2->std))
    printf("S_STD returned error %d\n",errno);

  ioctl(vid, VIDIOC_G_FMT, &v4l2->fmt);

  if (ioctl(vid, VIDIOC_S_FMT, &v4l2->fmt)) {
    printf("S_FMT returned error %d\n", errno);
    exit (1);
  }

  if (verbose) {
    printf("Captured Video : %dx%dx%d \"%4.4s\" images\n", 
	   v4l2->fmt.fmt.pix.width, v4l2->fmt.fmt.pix.height, 
	   v4l2->fmt.fmt.pix.depth, (char *)&v4l2->fmt.fmt.pix.pixelformat);
    printf("................ Each frame has %d bytes\n", v4l2->fmt.fmt.pix.sizeimage);
  }

  return vid;
}


/* *************************************************************************  
 * Function    : BV_get_vdoframe (int vid, char *outbuf, V4L2 *v4l2)
 * Inputs      : int vid      -  captured video image ID 
 *               char *outbuf -  buffer for captured image
 *               V4L2 *v4l2   -  io control parameters
 * Outputs     : size of captured image 
 * Description :
 * 
 *
 * ************************************************************************* */
int BV_get_vdoframe (int vid, char *outbuf, V4L2 *v4l2)
{
  struct v4l2_buffer	tempbuf;

  fd_set rdset;
  struct timeval timeout;
  int n;

  FD_ZERO(&rdset);
  FD_SET(vid, &rdset);
  timeout.tv_sec = 1;
  timeout.tv_usec = 0;

  n = select(vid + 1, &rdset, NULL, NULL, &timeout);

  if (n == -1)
    fprintf(stderr, "select error.\n");
  else if (n == 0)
    fprintf(stderr, "select timeout\n");
  else if (FD_ISSET(vid, &rdset)) {
    tempbuf.type = vimage.vidbuf.type;
    if(ioctl(vid, VIDIOC_DQBUF, &tempbuf))
      printf("DQBUF returned error %d\n", errno);
  }
}


int OLD_BV_get_vdoframe (int vid, char *outbuf, V4L2 *v4l2)
{
  int frame_size;
  
  frame_size = read(vid, outbuf, v4l2->fmt.fmt.pix.sizeimage);

  return frame_size;
}


