/* *************************************************************************
 * FILE   : XmDisplay.c
 * Author : Bo-kyun Na (bkna@npac.syr.edu) 
 * When   : 1999.09.28
 * All rights are reserved.
 * ************************************************************************* */

#include "vdo.h"

extern int verbose;

/* structure for menu buttons by Xt tool */
struct { int width, height; } capturesize[] = {
  {160, 120}, {176, 144}, {192, 144}, {320, 240}, {384, 288}, {640, 480}, {768, 576}, {1024, 768},
};


/* *************************************************************************  
 * Function    : frame_buffer_format(Display *d, Window w)
 * Inputs      :
 * Outputs     : __u32
 * Description :
 * ************************************************************************* */
__u32 frame_buffer_format (Display *d, Window w)
{
  __u32	fbf = V4L2_PIX_FMT_RGB565;

  XWindowAttributes wa;
  XGetWindowAttributes(d, w, &wa);

  if (wa.depth == 15) 
    fbf = V4L2_PIX_FMT_RGB555;
  else if (wa.depth == 16)
    fbf = V4L2_PIX_FMT_RGB565;
  else if (wa.depth == 24) 
    fbf = V4L2_PIX_FMT_BGR32;
  else if (wa.depth == 32) 
    fbf = V4L2_PIX_FMT_BGR32;
  else {
    printf("Unrecognized display depth. You may get an X Windows error.\n");
  }
  return fbf;
}


/* *************************************************************************  
 * Function    : quit(int vid, Widget w, XtPointer client, XtPointer call)
 * Inputs      :
 * Outputs     :
 * Description :
 * ************************************************************************* */
void quit(int vid, Widget w, XtPointer client, XtPointer call)
{
  struct v4l2_performance	perf;

  if (vid >= 0)	{
    if (ioctl(vid, VIDIOC_G_PERF, &perf) != 0)
      printf("G_PERF returned error code %d\n",errno);
    else
      printf("Captured %d frames\n", perf.frames);
    stop_capturing();
    close(vid);
  }
  exit(0);
}


/* *************************************************************************  
 * Function    : std_menu_callback()
 * Inputs      : int vid, Widget w, XtPointer client, XtPointer call
 * Outputs     :
 * Description :
 * ************************************************************************* */
void std_menu_callback(int vid, Widget w, XtPointer client, XtPointer call)
{
  StdMenuItem	*smi = (StdMenuItem *)client;

  stop_capturing();
  ioctl(vid, VIDIOC_S_STD, &smi->std);
  start_capturing();
}


/* *************************************************************************  
 * Function    : size_menu_callback()
 * Inputs      : int vid, Widget w, XtPointer client, XtPointer call
 * Outputs     :
 * Description :
 * ************************************************************************* */
void size_menu_callback(int vid, Widget w, XtPointer client, XtPointer call)
{
  int	i = (int)client;

  fmt.fmt.pix.width = capturesize[i].width;
  fmt.fmt.pix.height = capturesize[i].height;
  stop_capturing();
  ioctl(vid, VIDIOC_S_FMT, &fmt);
  start_capturing();
}


/* *************************************************************************  
 * Function    :
 * Inputs      :
 * Outputs     :
 * Description :
 * ************************************************************************* */
void BV_init_display (XWA *xwa, V4L2 *v4l2, int argc, char **argv, VIMAGE *vimage) 
{
  Widget w_toplevel;
  Widget w_topbox;
  Widget w_menubox;
  Widget w_quit;
  Widget w_stdbutton;
  Widget w_stdmenu;
  Widget w_sizebutton;
  Widget w_sizemenu;

  int i;

  fmt.fmt.pix.width = VDO_WIDTH;
  fmt.fmt.pix.height = VDO_HEIGHT;

  w_toplevel = XtAppInitialize (&app, "xvideum",
				NULL, 0, 
				&argc, argv, 
				NULL, NULL, 0);
  w_topbox = XtVaCreateManagedWidget ("topbox",
				     boxWidgetClass, w_toplevel,
				     XtNorientation, XtorientVertical,
				     XtNborderWidth, 0,
				     XtNhSpace, 0,
				     XtNvSpace, 0,
				     NULL);
  w_menubox = XtVaCreateManagedWidget ("menubox",
				      boxWidgetClass, w_topbox,
				      XtNorientation, XtorientHorizontal,
				      XtNborderWidth, 0,
				      XtNhSpace, 4,
				      XtNvSpace, 1,
				      NULL);

  w_stdbutton = XtVaCreateManagedWidget ("menubutton",
					menuButtonWidgetClass, w_menubox,
					XtNlabel, "Standard",
					XtNmenuName, "standard",
					XtNborderWidth, 0,
					NULL);
  w_stdmenu = XtVaCreatePopupShell ("standard", /* same as XtNmenuName & static */
				    simpleMenuWidgetClass, w_stdbutton,
				    NULL);
  for (i = 0; ; ++i) {
    struct v4l2_enumstd	estd;
    estd.index = i;
    err = ioctl(vid, VIDIOC_ENUMSTD, &estd);
    if (err)
      break;
    stditem[i].std = estd.std;
    stditem[i].w_item = XtVaCreateManagedWidget (estd.std.name, 
						smeBSBObjectClass, w_stdmenu,
						 XtNlabel, estd.std.name, NULL);
    XtAddCallback (stditem[i].w_item, XtNcallback, std_menu_callback, &stditem[i]);
  }

  w_sizebutton = XtVaCreateManagedWidget ("menubutton",
					  menuButtonWidgetClass, w_menubox,
					  XtNlabel, "Zoom",
					  XtNmenuName, "Zoom",
					  XtNborderWidth, 0, NULL);
  w_sizemenu = XtVaCreatePopupShell ("Zoom", /* same as XtNmenuName & static */
				     simpleMenuWidgetClass, w_sizebutton, NULL);
  for (i = 0; i < sizeof(capturesize)/sizeof(capturesize[0]); ++i) {
    char sizestr[32];
    Widget w;

    sprintf(sizestr, "%d x %d", capturesize[i].width, capturesize[i].height);
    w = XtVaCreateManagedWidget (sizestr, smeBSBObjectClass, w_sizemenu,
				 XtNlabel, sizestr, NULL);
    XtAddCallback (w, XtNcallback, size_menu_callback, (char *)i);
  }

  w_quit = XtVaCreateManagedWidget ("Quit!", commandWidgetClass, w_menubox,
				    XtNborderWidth, 0, NULL);
  XtAddCallback (w_quit, XtNcallback, quit, NULL);
  w_viewport = XtVaCreateManagedWidget ("port",
					portholeWidgetClass, w_topbox,
					XtNwidth, VDO_WIDTH,
					XtNheight, VDO_HEIGHT,
					NULL);
  XtRealizeWidget(w_toplevel);
  imagegc = XCreateGC(XtDisplay(w_viewport), XtWindow(w_viewport), 0, NULL);
  if (imagegc == NULL) {
    printf("XCreateGC failed.\n");
    return 1;
  }

}


/* *************************************************************************
 * int BV_init_display ()
 *
 *
 * ************************************************************************* */
void OLD_BV_init_display (XWA *xwa, V4L2 *v4l2, int argc, char **argv, VIMAGE *vimage) 
{
  Widget toplevel;
  Widget topbox;
  Widget menubox;
  Widget quit_widget;

  /* Set the size of display */
  v4l2->fmt.fmt.pix.width  = VDO_WIDTH;
  v4l2->fmt.fmt.pix.height = VDO_HEIGHT;

  toplevel = XtAppInitialize(&xwa->app, "IARI", NULL, 0, &argc, argv, NULL, NULL, 0);
  topbox = XtVaCreateManagedWidget("topbox", boxWidgetClass, toplevel,
			    XtNorientation, XtorientVertical, XtNborderWidth,
			    0, XtNhSpace, 0, XtNvSpace, 0, NULL);
  menubox = XtVaCreateManagedWidget("menubox", boxWidgetClass, topbox,
			    XtNorientation, XtorientHorizontal, XtNborderWidth,
			    0, XtNhSpace, 4, XtNvSpace, 1, NULL);
  quit_widget = XtVaCreateManagedWidget("Close!", commandWidgetClass, menubox,
			    XtNborderWidth, 0, NULL);
  XtAddCallback(quit_widget, XtNcallback, (XtCallbackProc) exit, NULL);
  xwa->viewport = 
    XtVaCreateManagedWidget("port", portholeWidgetClass, topbox, XtNwidth,
			   v4l2->fmt.fmt.pix.width, XtNheight, 
			    v4l2->fmt.fmt.pix.height, NULL);
  XtRealizeWidget(toplevel);

#ifdef PIXELFORMAT
  v4l2->fmt.fmt.pix.pixelformat = PIXELFORMAT;
#else
  /* Get display pixel format from the info of the graphic video card  */
  v4l2->fmt.fmt.pix.pixelformat = frame_buffer_format(XtDisplay(xwa->viewport),
		                  DefaultRootWindow(XtDisplay(xwa->viewport)));
#endif
  if (verbose)
    printf ("Display pixel format is %4.4s and the depth is %d\n",
	    (char *)&v4l2->fmt.fmt.pix.pixelformat, xwa->xdepth);
  
  xwa->gc = XCreateGC(XtDisplay(xwa->viewport),XtWindow(xwa->viewport),0,NULL);
  if (xwa->gc == NULL) {
    printf("XCreateGC failed.\n");
    exit (1);
  }

  v4l2->fmt.fmt.pix.sizeimage = VDO_WIDTH * VDO_HEIGHT * (xwa->xdepth / 8);

  vimage->data = calloc(1, v4l2->fmt.fmt.pix.sizeimage);
  if (vimage->data == NULL) {
    printf("malloc(%d) failed\n", v4l2->fmt.fmt.pix.sizeimage);
    exit (1);
  }

  vimage->ximage = XCreateImage (XtDisplay(xwa->viewport), 
				 DefaultVisual(XtDisplay(xwa->viewport), 0),
				 (xwa->xdepth == 32) ? 24 : xwa->xdepth,ZPixmap,
				 0, vimage->data, v4l2->fmt.fmt.pix.width,
				 v4l2->fmt.fmt.pix.height, 16, 0);

  if (vimage->ximage == NULL) {
    printf("No XImage\n");
    exit (1);
  }
}


/* *************************************************************************  
 * Function    : BV_display_vdoframe (XWA *xwa, VIMAGE *vimage, V4L2 *v4l2)
 * Inputs      :
 * Outputs     :
 * Description :
 * ************************************************************************* */
void BV_display_vdoframe (XWA *xwa, VIMAGE *vimage, V4L2 *v4l2)
{
  XEvent event;

  err = 0;
  if (XtIsRealized(w_viewport)) {
      XPutImage(XtDisplay(w_viewport), XtWindow(w_viewport), imagegc,
		vimage.ximage, 0, 0, 0, 0,
		fmt.fmt.pix.width, fmt.fmt.pix.height);
  }
  else {
    XtAppPeekEvent(app, &event);
  }

  while (XtAppPending(app)) {
    XtAppNextEvent(app, &event);
    switch (event.type) {
    default:
      break;
    }
    XtDispatchEvent(&event);
  }
}

/* *************************************************************************
 * int BV_display_vdoframe ()
 *
 *
 * ************************************************************************* */
void OLD_BV_display_vdoframe (XWA *xwa, VIMAGE *vimage, V4L2 *v4l2)
{
  XEvent event;

  if (XtIsRealized(xwa->viewport)) 
    XPutImage (XtDisplay(xwa->viewport), XtWindow(xwa->viewport), xwa->gc,
	       vimage->ximage, 0, 0, 0, 0, 
	       v4l2->fmt.fmt.pix.width, v4l2->fmt.fmt.pix.height); 

  else 
    XtAppPeekEvent(xwa->app, &event);

  while (XtAppPending(xwa->app)) {
    XtAppNextEvent(xwa->app, &event);
    XtDispatchEvent(&event);
  }

}


#ifdef CODEC
/* *************************************************************************
 * char *BV_vdo_code (int codec_type, char *input)
 *
 *
 * ************************************************************************* */
char *BV_vdo_code (int codec_type, char *input)
{
  static int first = 1;
  int size;
  char buf[1000];

  switch (codec_type) {
  case H261:
    if (first) {
      size = H261codeIntraFrame (input, buf);
      first = 0;
    } 
    else 
      size = H261codeFrame (&input, buf);
    return buf;

    break;
  case H263: {
    if (first) {
      size = H263codeIntraFrame (input, buf);
      first = 0;
    } 
    else 
      size = H263codeFrame (&input, buf);
    return buf;
  }
  case MPEG:
    break;
  default:
    break;
  }

}


/* *************************************************************************
 * char *BV_vdo_decode (int codec_type, char *input, int size)
 *
 *
 * ************************************************************************* */
char *BV_vdo_decode (int codec_type, char *input, int size)
{
  char *frame;

  switch (codec_type) {
  case H261:
    break;
  case H263: 
    frame = (char *) H263decodeFrame (0, input, size);
    return frame;
  case MPEG:
    break;
  default:
    break;
  }

}

#endif
