
/* *************************************************************************
 * File : colormap.c (convert rgb16 pixel format to yuv411)
 * by   : Bo-kyun Na, IntellActive Research.
 *        bkna@npac.syr.edu
 * Date : 1999.08.24
 * ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "config.h"
#include "tmndec.h"
#include "global.h"

/* Data for ConvertYUVtoRGB*/
// #if (defined DISPLAY || defined WINDOWS)
long int crv_tab[256];
long int cbu_tab[256];
long int cgu_tab[256];

long int cgv_tab[256];
long int tab_76309[256];


/* *************************************************************************
 * Function    : init_dither_tab()
 *
 *
 * ************************************************************************* */
void init_dither_tab()
{
  long int crv,cbu,cgu,cgv;
  int i;   
  
  crv = 104597; cbu = 132201;  /* fra matrise i global.h */
  cgu = 25675;  cgv = 53279;
  
  for (i = 0; i < 256; i++) {
    crv_tab[i] = (i-128) * crv;
    cbu_tab[i] = (i-128) * cbu;
    cgu_tab[i] = (i-128) * cgu;
    cgv_tab[i] = (i-128) * cgv;
    tab_76309[i] = 76309*(i-16);
  }
}


/* *************************************************************************
 * Function    : RGB2YUV ()
 * Description : Convert pixel format from RGB16 to YUV411.
 * Input       : input - pt. of input buffer
 *               y,u,v - pt.s of out buffers for lum, cr, and cb.
 *               width, height - image sizes 
 * Return      : nothing
 * Date        :
 * ************************************************************************* */
void RGB16toYUV(unsigned char *input, unsigned char *y, unsigned char *u, 
		unsigned char *v, int width, int height)
{
  register int i, j;
  register unsigned char *in;
  register short cru, crv, lum;
  register unsigned char *y1 = y, *u1 = u, *v1 = v; 
  const int QCIFwidth = 176;
  const int QCIFheigth = 144;
  static int first = 1;
  static float  mult299[1024], mult587[1024], mult114[1024];
  static float  mult16874[1024], mult33126[1024], mult5[1024];
  static float mult41869[1024], mult08131[1024];
  register unsigned char r, g, b;

  in = input + width * (height - 1) * 2;

  if (first) {
    register int index;
    register int maxValue;

    maxValue = 256;

    for (index = 0; index <= maxValue; index++) {
      mult299[index] = index * 0.29900;
      mult587[index] = index * 0.58700;
      mult114[index] = index * 0.11400;
      mult16874[index] = -0.16874 * index;
      mult33126[index] = -0.33126 * index;
      mult5[index] = index * 0.50000;
      mult41869[index] = -0.41869 * index;
      mult08131[index] = -0.08131 * index;
    }
 
    first = 0;
  }

  for(i = 0; i < QCIFheigth; i++) {
    for(j = 0; j < QCIFwidth; j++) {
      b = (*(in) & 31) << 3;
      g = ((*(in) & 224) >> 2) | (*(in + 1) & 3) << 6;
      r = (*(in + 1) & 124) << 1;

      lum =  mult114[b] + mult587[g] + mult299[r];
   
      *(y1++) = (219 * lum / 255) + 16;

      if(((i % 2) == 1) && ((j % 2) == 1)) {
	cru =  mult5[b] + mult33126[g] + mult16874[r];

	*(u1++) = (224 * cru / 255) + 128;

	crv =  mult08131[b] + mult41869[g] + mult5[r];

	*(v1++) = (224 * crv / 255) + 128;
      }

      in += 2;
    }
    in -= 2 * (width + QCIFwidth);
  }
}


            
/**********************************************************************
 *
 *	Name:	         ConvertYUVtoRGB	
 *	Description:     Converts YUV image to RGB (packed mode)
 *	
 *	Input:	         pointer to source luma, Cr, Cb, destination,
 *                       image width and height
 *	Returns:       
 *	Side effects:
 *
 *	Date: 951208	Author: Karl.Lillevold@nta.no
 *
 ***********************************************************************/
void ConvertYUVtoRGB (unsigned char *src0, unsigned char *src1, 
 	    unsigned char *src2, unsigned char *dst_ori, int width, int height)
{       
  extern long int crv_tab[];
  extern long int cbu_tab[];
  extern long int cgu_tab[];

  extern long int cgv_tab[];
  extern long int tab_76309[];

  int y11,y21;
  int y12,y22;
  int y13,y23;
  int y14,y24;
  int u,v; 
  int i,j;
  int c11, c21, c31, c41;
  int c12, c22, c32, c42;
  unsigned int DW;
  unsigned int *id1, *id2;
  unsigned char *py1,*py2,*pu,*pv;
  unsigned char *d1, *d2;
  
  d1 = dst_ori;
  d1 += width*height*3 - width*3;
  d2 = d1 - width*3;
  
  py1 = src0; pu = src1; pv = src2;
  py2 = py1 + width;
 
  id1 = (unsigned int *)d1;
  id2 = (unsigned int *)d2;

  for (j = 0; j < height; j += 2) { 
    /* line j + 0 */
    for (i = 0; i < width; i += 4) {
      u = *pu++;
      v = *pv++;
      c11 = crv_tab[v];
      c21 = cgu_tab[u];
      c31 = cgv_tab[v];
      c41 = cbu_tab[u];
      u = *pu++;
      v = *pv++;
      c12 = crv_tab[v];
      c22 = cgu_tab[u];
      c32 = cgv_tab[v];
      c42 = cbu_tab[u];

      y11 = tab_76309[*py1++]; /* (255/219)*65536 */
      y12 = tab_76309[*py1++];
      y13 = tab_76309[*py1++]; /* (255/219)*65536 */
      y14 = tab_76309[*py1++];

      y21 = tab_76309[*py2++];
      y22 = tab_76309[*py2++];
      y23 = tab_76309[*py2++];
      y24 = tab_76309[*py2++];

      /* RGBR*/
      DW = ((clp[(y11 + c41)>>16])) |
           ((clp[(y11 - c21 - c31)>>16])<<8) |
	   ((clp[(y11 + c11)>>16])<<16) |  
	   ((clp[(y12 + c41)>>16])<<24);
      *id1++ = DW;

      /* GBRG*/
      DW = ((clp[(y12 - c21 - c31)>>16])) |
           ((clp[(y12 + c11)>>16])<<8) |  
           ((clp[(y13 + c42)>>16])<<16) |
           ((clp[(y13 - c22 - c32)>>16])<<24);
      *id1++ = DW;

      /* BRGB*/
      DW = ((clp[(y13 + c12)>>16])) |  
           ((clp[(y14 + c42)>>16])<<8) |
           ((clp[(y14 - c22 - c32)>>16])<<16) |
           ((clp[(y14 + c12)>>16])<<24);  
      *id1++ = DW;

      /* RGBR*/
      DW = ((clp[(y21 + c41)>>16])) |
           ((clp[(y21 - c21 - c31)>>16])<<8) |
           ((clp[(y21 + c11)>>16])<<16) |  
           ((clp[(y22 + c41)>>16])<<24);
      *id2++ = DW;

      /* GBRG*/
      DW = ((clp[(y22 - c21 - c31)>>16])) |
           ((clp[(y22 + c11)>>16])<<8) |  
           ((clp[(y23 + c42)>>16])<<16) |
           ((clp[(y23 - c22 - c32)>>16])<<24);
      *id2++ = DW;

      /* BRGB*/
      DW = ((clp[(y23 + c12)>>16])) |  
           ((clp[(y24 + c42)>>16])<<8) |
           ((clp[(y24 - c22 - c32)>>16])<<16) |
           ((clp[(y24 + c12)>>16])<<24);  
      *id2++ = DW;
    }
    id1 -= (9 * width)>>2;
    id2 -= (9 * width)>>2;
    py1 += width;
    py2 += width;
  }           
}  

// #endif
