
/* =========================================================================
 * FILE       : RxODO.c
 * -----------:-------------------------------------------------------------
 * PROGRAMMER : Bo-kyun Na (bkna@npac.syr.edu), IntellActive Research Inc.
 *            : 1999.10.12.
 * -----------:-------------------------------------------------------------
 * ********** :     All rights are reserved by Bo-kyun Na.        **********
 * -----------:-------------------------------------------------------------
 * Revision   : 
 * -----------:-------------------------------------------------------------
 * This example receives compressed audio data over the local network,
 * plays audio data by /dev/dsp device.
 * ========================================================================= */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>		/* only for linux */
#include <malloc.h>
#include <sys/types.h>
#include <sys/param.h>

#include "network.h"
#include "params.h"
#include "transceive.h"

#include "odo.h"

int verbose = 0;



/* *************************************************************************
 * Function    : main ()
 * ------------:------------------------------------------------------------
 * Inputs      : none
 * Outputs     : int 
 * Description : void *Thread_TxAudio (void *arg)
 * ************************************************************************* */
int main(int argc, char **argv)
{
  int fd;			/* sound device file descriptor */
  int status;			/* return status of system calls */
  int wr;			/* size in byte unit */
  int odo_frame;

  char dev_odo[64];		/* device driver of audio device */
  unsigned char *buf;		/* this buffer holds the digitized audio */

  Parameters *param;
  TransceiveState *RxState;
  MsgType *message;

  /* audio device driver */
  strcpy (dev_odo, ODO_DEVICE);		

  odo_frame = ODO_LENGTH * ODO_RATE * ODO_SIZE * ODO_CHANNELS / 8;
  buf = (unsigned char *) malloc (odo_frame);
  if (buf == NULL) {
    printf ("malloc fail: buf = %d\n", odo_frame);
    exit (1);
  }

  message = (MsgType *) malloc (sizeof (MsgType));
  if (message == NULL) {
    printf ("malloc fail: message = %d\n", sizeof (MsgType));
    exit (1);
  }

  /* Initialization of receiver parameters */
  param = (Parameters *) init_Param (argc, argv);

  /* Initialization of audio */
  fd = BV_ODO_initialize (dev_odo);
  printf ("Initialized audio devices\n");

  /* Init. of network */
  RxState = (TransceiveState *) initialize_receive (param, AUDIO);

  /* loop for audio play until Control-C */
  for ( ; ; ) {		
    RxState->session = 0;
    message =  (MsgType *) BV_ODO_receive (RxState, message);
    //    memcpy (buf, message.data, odo_frame);

    wr = BV_ODO_play (fd, message->data, odo_frame);
    printf ("Play size is %d\n", wr);

    /* wait for playback to complete before recording again */
    status = ioctl(fd, SNDCTL_DSP_SYNC, 0); 
    if (status == -1)
      perror("SOUND_PCM_SYNC ioctl failed");
  }
}

