/* =========================================================================
 * FILE       : TxODO.c
 * -----------:-------------------------------------------------------------
 * PROGRAMMER : Bo-kyun Na (bkna@npac.syr.edu), IntellActive Research Inc.
 *            : 1999.10.12.
 * -----------:-------------------------------------------------------------
 * ********** :     All rights are reserved by Bo-kyun Na.        **********
 * -----------:-------------------------------------------------------------
 * Revision   : 
 * -----------:-------------------------------------------------------------
 * This example records audio from /dev/dsp device, transmits compressed
 * audio over the local network.
 * ========================================================================= */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>		/* only for linux */
#include <malloc.h>
#include <sys/types.h>
#include <sys/param.h>

#include "network.h"
#include "params.h"
#include "transceive.h"
#include "odo.h"

int verbose = 0;

extern int transmit_packet (TransceiveState *tstate, PacketType *packet);
void BV_odo_packetize (unsigned char *data, int msg_sz, 
		       TransceiveState *tstate, Parameters *param);


/* *************************************************************************
 * Function    : main ()
 * ------------:------------------------------------------------------------
 * Inputs      : none
 * Outputs     : int 
 * Description : void *Thread_TxAudio (void *arg)
 * ************************************************************************* */
int main(int argc, char **argv)
{
  int fd, fdw;			/* sound device file descriptor */
  int status;			/* return status of system calls */
  int rd;			/* size in byte unit */
  int odo_frame;

  char dev_odo[64];		/* device driver of audio device */
  unsigned char *buf;		/* this buffer holds the digitized audio */

  /* Network parameters */
  Parameters *param;
  TransceiveState *tstate;


  /* audio device driver */
  strcpy (dev_odo, ODO_DEVICE);		

  odo_frame = ODO_LENGTH * ODO_RATE * ODO_SIZE * ODO_CHANNELS / 8;
  buf = (unsigned char *) malloc(odo_frame);
  if (buf == NULL) {
    printf ("malloc fail: sizeof (buf) = %d\n", odo_frame);
    exit (1);
  }

  /* Initialization of network parameters */
  param = (Parameters *) init_TxVidParam (argc, argv);

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, AUDIO);

  /* Initialization of audio */
  //  fd = BV_ODO_initialize (dev_odo);

  for ( ; ; ) {			/* loop until Control-C */
    fd = BV_ODO_initialize (dev_odo);
    rd = BV_ODO_record (fd, buf, odo_frame);
    close (fd);
    printf ("Recorded audio size is %d bytes\n", rd);

    BV_odo_packetize (buf, odo_frame, tstate, param);

    fdw = BV_ODO_initialize (dev_odo);
    BV_ODO_play (fdw, buf, odo_frame);

    /* wait for playback to complete before recording again */
    //    status = ioctl(fdw, SNDCTL_DSP_SYNC, 0); 
    /* stop playback immediately to complete before recording again */
    status = ioctl(fdw, SNDCTL_DSP_RESET, 0); 
    /* pause playback to complete before recording again */			
    //    status = ioctl(fd, SNDCTL_DSP_POST, 0); 
    if (status == -1)
      perror("SNDCTL_DSP_RESET ioctl failed");
    close (fdw);
  }
}
