
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"

#define BUFFER_SIZE (1024*10)
#define STREAMBUFS 4

struct pollfd pollfds[2];	/* polling file descriptors for socket */

extern int verbose;
extern int input_fd, output_fd;



/* ********************************************************************
 * Thread : void *Thread_RxVideo(void *arg)
 *
 *
 * ******************************************************************** */
void main (int argc, char **argv)
{
  int xdepth = 0;
  int CISsize=0, first=1, userno = 0;
  uchar *y, *u, *v;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 
  V4L2 v4l2;
  VIMAGE vimage[STREAMBUFS];
  Widget viewport;
  XtAppContext	app;
  GC gc_recvd_img;

  int packetcnt = 0;
  int currentframe;
  int lastw, lasth;
  int i;
  unsigned int currentsession;
  unsigned short myflags;

  Parameters *params;
  TransceiveState *receiver;
  PacketType packets[MAXPACKETS];	/* circular buffer of rec'vd packets */
  PacketType *packetlist[MAXPACKETS];	/* packets for this frame */
  PacketType *curpacket;
  MsgType message;

  /* .... initialize everything ....  */
  params = (Parameters *) init_RxVidParam(argc, argv);
  receiver = initialize_receive(params, VIDEO);

  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  receiver->session = 1;	/* choose remote host to receive */
  v4l2.req.count = 1;		/* Num of requested window displays */

  /* 
  (char *) receiver->name[receiver->session].sin_addr = 
                                  (char *) params->hostname[receiver->session];
  printf ("remotehost = %s\n", receiver->name[receiver->session]);
  */

  /* Initialization of image display  */
  gc_recvd_img = 
    BV_init_display (&app, &xdepth, &viewport, &v4l2, argc, argv, vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_decoder ();		/* Initilization of H261 decoder */

  /* .... receive video ....      */
  currentframe = 0;
  currentsession = 0;
  lastw = params->width;
  lasth = params->height;
   
  myflags = 0xffff;
  for (i = 0; i < MAXPACKETS; i++) packetlist[i] = NULL;

  for (;;) {
    //    sem_wait (&VR);
    //    sem_post (&VS); 

    //    fprintf (stderr, "RecvVidThread ");

    curpacket = &packets[packetcnt++];
    if (packetcnt >= MAXPACKETS) packetcnt = 0;

    /* Get video packet from remote if available. */
    CISsize += receive_packet(receiver, curpacket);
    if (CISsize == 0) {
      //    thrd_stop (&AR); 
      continue;
    }

    if (currentsession != curpacket->header.session) {
      /* if you join a conference in progress then the current frame and 
       * session will be grabbed from the first received packet received.  */
      if (verbose) {
	fprintf(stderr, "changing session from %d to %d\n",
		currentsession, curpacket->header.session);
      }
      for (i = 0; i < MAXPACKETS; i++) 	packetlist[i] = NULL;
      myflags = 0xffff;
      currentframe = curpacket->header.frame;
      currentsession = curpacket->header.session;
    }

    /* Resize of window :
     * if this is the first packet of a frame, check for a resize.
     * In BuenaVista, it should be removed. (x and y in packe header).
     */
    if ((curpacket->header.flags & 1) == 0) {
      /* the size of each frame is sent on the first packet of each frame */
      if ((curpacket->header.x != lastw) || (curpacket->header.y != lasth)) {
	lastw = curpacket->header.x;
	lasth = curpacket->header.y;

	if (verbose) 
	  printf("resizing video %dx%d\n",lastw, lasth);

	/* Resize of window */
	/*	resize_window (window, lastw, lasth); */
      }
    }

    if (curpacket->header.frame < currentframe) {
      /* if this packet is from an earlier frame, disregard it. */
      if (verbose)
	fprintf(stderr, "received stale packet %d, current frame %d\n",
		curpacket->header.frame, currentframe);
      //    thrd_stop (&AR); 
      continue;
    }

    if (curpacket->header.frame > currentframe) {
      /* If the frame number increased then we probably missed the last
       * packet of the previous frame. We display what we can drop of the
       * previous frame if '-b' option for verbose was selected, otherwise 
       * we just drop the partial frame.                                  
       */
      if (myflags != 0) {
	if (verbose)
	  fprintf(stderr, "dropping %d due to getting packet %d (flags %x)\n",
		  currentframe, curpacket->header.frame, myflags);
      }

      for (i = 0; i < MAXPACKETS; i++) packetlist[i] = NULL;
      myflags = 0xffff;
      currentframe = curpacket->header.frame;
    }

    /* Save the packet for decoding */
    packetlist[packetnum(curpacket->header.flags)] = curpacket;
    myflags &= curpacket->header.flags;

    if (myflags == 0) {
      if (params->do_cis == H261) {
	//	create_message_h261 (&message, packetlist);
	create_message (&message, packetlist);
      }
      else 
	create_message(&message, packetlist);

      /* just for test of h261    -- bkna */
      //      create_message(&message, packetlist);
      //      vimage[1].data = &message;

      printf ("Rx:encoded message size = %d\n", message.header.size);
      decode_frame (inbuf, message.header.size, message.data, first, userno);
      //    YUVtoRGB(vimage[1].data, y, u, v, VDO_WIDTH, VDO_HEIGHT);

      /* Display images of converted image pixel format */
      //      BV_display_vdoframe (&viewport, &vimage[1], &gc_recvd_img, &v4l2, &app);

      /* Display images before conversion of image pixel format */
      BV_display_vdoframe (&viewport, inbuf, &gc_recvd_img, &v4l2, &app);

      /* Display raw image data */
      //      vimage[0].data = curpacket->payload; 
      //      BV_display_vdoframe (&viewport, &vimage[0], &gc_recvd_img, &v4l2, &app);
    }

    //    thrd_stop (&AR); 
  }
}


