
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"

struct pollfd pollfds[2];	/* polling file descriptors for socket */
unsigned char *clp;

int verbose = 0;


/* ********************************************************************
 * Thread : void *Thread_RxVideo(void *arg)
 *
 *
 * ******************************************************************** */
int main (int argc, char **argv)
{
  FILE *df;
  char datafile[80];

  int i;
  int userno = 0;
  uchar *y, *u, *v;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 

  V4L2 v4l2;
  XWA xwa;
  VIMAGE vimage;

  Parameters *param;
  TransceiveState *receiver;
  MsgType *message;


  xwa.xdepth = 16;
  v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB565;

  /* .... initialize everything ....  */
  message = (MsgType *) malloc (sizeof (MsgType));
  if (message == NULL) {
    printf ("malloc fail: message = %d\n", sizeof (MsgType));
    exit (1);
  }

  param = (Parameters *) init_Param(argc, argv);
  receiver = initialize_receive(param, VIDEO);

  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  receiver->session = 1;	/* choose remote host to receive */
  param->codec = RAW;


  /* Process command line arguments */
  for (i = 1; i < argc; i++) {
    if (!strcmp (argv[i], "-C")) {
      if (!strcmp(argv[++i], "H261")) {
      	param->codec = H261;
      	param->cis_type = "H261";
      }
      else if (!strcmp(argv[i], "H263")) {
      	param->codec = H263;
      	param->cis_type = "H263";
      }
      else if (!strcmp(argv[i], "JPEG")) {
      	param->codec = JPEG;
      	param->cis_type = "Jpeg";
      }
      else if (!strcmp(argv[i], "MPEG")) {
      	param->codec = MPEG;
      	param->cis_type = "Mpeg1";
      }
      else if (!strcmp(argv[i], "RAW")) {
	param->codec = RAW;
      	param->cis_type = "RAW";
      }
      else {
	printf ("Specifiy your codec mode\n");
	exit (0);
      }
    }
    /*    if (!strcmp(argv[i], "-i")) {
      strcpy (fname, argv[++i]);
      readable = 1;
      fp = fopen(fname, "r");
      if (fp == NULL) printf ("Can't open image file: %s\n", fname);
      else  printf ("Image file name is %s\n", fname);
    }
    if (!strcmp(argv[i], "-o")) {
      strcpy (fname, argv[++i]);
      writable = 1;
      fp = fopen(fname, "w");
      if (fp == NULL) printf ("Can't open image file: %s\n", fname);
      else  printf ("Image file name is %s\n", fname);
    }
    */
    if(!strcmp(argv[i], "-r")) {
      //      rate = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-s")) {
      //      width = atoi(argv[++i]);
      //      height = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-v")) {
      verbose = 1;
    }
    if(!strcmp(argv[i], "-x")) {
      xwa.xdepth = atoi(argv[++i]);
      if (xwa.xdepth == 15)
	v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB555;
      else if (xwa.xdepth == 16)
	v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB565;
      else if (xwa.xdepth == 24)
	v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR24;
      else if (xwa.xdepth == 32)
	v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR32;
      else {
	printf("Unrecognized display depth. Type correct depth\n");
	exit (0);
      }
    }
  }

  printf ("codec is %s\n", param->cis_type);

  /* Initialization of image display  */
  vimage.data = BV_init_display (&xwa, &v4l2, argc, argv, &vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_decoder ();		/* Initilization of H261 decoder */

  /* clip table */
  if (!(clp = (unsigned char *) malloc(1024))) error("malloc failed\n");
  clp += 384;
  for (i=-384; i<640; i++) clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

  //  if (param->rdfile != '\0') {
  //    strcpy (datafile, "vimage.raw");
  //    printf ("image file name is %s\n", datafile);
  //    df = fopen(datafile, "w");
  //  }


  for (;;) {
    //    sem_wait (&VR);
    //    sem_post (&VS); 
    //    fprintf (stderr, "RecvVidThread ");

    message = (MsgType *) BV_VDO_receive (receiver, message);

    if (param->codec == H261) {
      /* Decode of recvd message and conversion to RGB from YUV  */
	decode_frame (message->data, message->header.size, inbuf, 
		      ((message->header.frame % 10) == 0) ? 1 : 0, userno);
	ConvertYUVtoRGB(y, u, v, vimage.data, VDO_WIDTH, VDO_HEIGHT);
	//	memcpy(vimage.data, message->data, message->header.size); 
      } 
      else if (param->codec == H263) {
	
      }
      else if (param->codec == JPEG) {

      }
      else if (param->codec == MPEG) {

      }
      else if (param->codec == RAW) {
	memcpy(vimage.data, message->data, message->header.size); 
      }
      else {
	printf ("Error on creating message. Specify the codec type\n");
	exit (0);
      }

      //      if (verbose)
	printf ("Rx: The %dth image frame w/ size = %d bytes\n", 
		message->header.frame, message->header.size);

      /* Display images */
      BV_display_vdoframe (&xwa, &vimage, &v4l2);

      /* Store received video data as a file */
      //      if (param->wrfile != '\0')	
      //	fwrite(vimage.data, 1, message->header.size, df);

    //    thrd_stop (&AR); 
  }
}


