
/* *************************************************************************
 *  File : MTvodo.c
 * -------------------------------------------------------------------------
 *  All rights are reserved by Bo-kyun Na, IntellActive Research Inc.
 * -------------------------------------------------------------------------
 * This example reads an image from /dev/video, transmits compressed video 
 * over the local network, and optionally displays the image in Xwindow.
 * ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"

struct pollfd pollfds[2];	/* polling file descriptors for socket */
unsigned char *clp;

unsigned long wpixel[3];
int expand;
extern int verbose;


/* *************************************************************************
 * void *Thread_TxVideo (void *arg)
 *
 *
 * ************************************************************************* */
void main (int argc, char **argv)
{
  FILE *fp;
  char fname[80];
  int readable = 0, writable = 0;

  int i;

  Parameters *param;
  TransceiveState *tstate;

  int vid;
  char	devname[64];
  unsigned char *capimg;
  XWA xwa;
  V4L2 v4l2;
  VIMAGE vimage;

  int first = 0;
  int CISsize;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 
  char outbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  char tmpbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  unsigned char *y, *u, *v;
  unsigned char *y1, *u1, *v1;
  int userno = 0;

  /* Put in the device node name */
  strcpy(devname, VDO_DEVICE);

  /* Initialization of video devices */
  param = (Parameters *) init_TxVidParam (argc, argv);

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, VIDEO);

  /* Pointers for the conversion of RGBtoYUV */
  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  y1 = tmpbuf;
  u1 = &tmpbuf[VDO_WIDTH * VDO_HEIGHT];
  v1 = &tmpbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  xwa.xdepth = 32;		/* init. of depth on xwindow */      

  /* Process command line arguments */
  for (i = 1; i < argc; i++) {
    if (!strcmp (argv[i], "-C")) {
      if (!strcmp(argv[++i], "H261")) {
      	param->codec = H261;
      	param->cis_type = "H261";
      }
      else if (!strcmp(argv[i], "H263")) {
      	param->codec = H263;
      	param->cis_type = "H263";
      }
      else if (!strcmp(argv[i], "JPEG")) {
      	param->codec = JPEG;
      	param->cis_type = "Jpeg";
      }
      else if (!strcmp(argv[i], "MPEG")) {
      	param->codec = MPEG;
      	param->cis_type = "Mpeg1";
      }
      else if (!strcmp(argv[i], "RAW")) {
	param->codec = RAW;
      	param->cis_type = "RAW";
      }
      else {
	printf ("Specifiy your codec mode\n");
	exit (0);
      }
    }
    if(!strcmp(argv[i], "-D")) {
      strcpy (devname, argv[++i]);
    }
    if (!strcmp(argv[i], "-i")) {
      strcpy (fname, argv[++i]);
      readable = 1;
      fp = fopen(fname, "r");
      if (fp == NULL) printf ("Can't open image file: %s\n", fname);
      else  printf ("Image file name is %s\n", fname);
    }
    if (!strcmp(argv[i], "-o")) {
      strcpy (fname, argv[++i]);
      writable = 1;
      fp = fopen(fname, "w");
      if (fp == NULL) printf ("Can't open image file: %s\n", fname);
      else  printf ("Image file name is %s\n", fname);
    }
    if(!strcmp(argv[i], "-r")) {
      //      rate = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-s")) {
      //      width = atoi(argv[++i]);
      //      height = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-v")) {
      verbose = 1;
    }
    if(!strcmp(argv[i], "-x")) {
      xwa.xdepth = atoi(argv[++i]);
    }
  }

  printf ("codec is %s\n", param->cis_type);

  /* Function: Init. of video capture to get the video from my camera */
  vid = BV_init_vidcap (devname, &v4l2);
  capimg = (unsigned char *) malloc (v4l2.fmt.sizeimage);

  /* Init. and format of xwindow display */
  BV_init_display (&xwa, &v4l2, argc, argv, &vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_encoder ();		/* Initilization of H261 encoder */
  init_decoder ();

  if (xwa.xdepth == 32 || xwa.xdepth == 24 || xwa.xdepth == 16) {
    XWindowAttributes wa;

    XGetWindowAttributes(XtDisplay(xwa.viewport),
			 DefaultRootWindow(XtDisplay(xwa.viewport)), &wa);

    wpixel[0] = wa.visual->red_mask;
    wpixel[1] = wa.visual->green_mask;
    wpixel[2] = wa.visual->blue_mask;

    /* If the colors in 16/24/32-bit mode are wrong, try this instead
       of the above three lines */
    /*
    wpixel[2] = wa.visual->red_mask;
    wpixel[1] = wa.visual->green_mask;
    wpixel[0] = wa.visual->blue_mask;
    */

    InitColorDither(xwa.xdepth == 24 || xwa.xdepth == 32, &xwa);
  }
  else {
    //      ord4x4_dither_init ();
    printf ("Wrong window depth\n");
  }


  /* clip table */
  if (!(clp = (unsigned char *) malloc(1024))) error("malloc failed\n");
  clp += 384;
  for (i=-384; i<640; i++) clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

  /* Loop processing related to a video frame */
  for (;;) {
    //    sem_wait (&VS);
    //    fprintf (stderr, "SendVidThread ");

    if (readable)
      fread (vimage.data, v4l2.fmt.sizeimage, 1, fp);
    else
      BV_get_vdoframe (vid, capimg, &v4l2);

    /* Packetization and sending of video data */
    if (param->transmit) {
      if (param->codec == H261) {
	/* Conversion image pixel format */
	RGB16toYUV(capimg, y, u, v, VDO_WIDTH, VDO_HEIGHT);

	CISsize = ((first %= 10) == 0) ? 
	          p64EncodeIntraFrame (inbuf, outbuf):
	          p64EncodeFrame (inbuf, outbuf);
 
	/* transmit compressed image sequence data */
	BV_packetize_vdoframe(outbuf,CISsize,VDO_WIDTH,VDO_HEIGHT,tstate,param);

	/* Decode of recvd message and conversion to RGB from YUV  */
	decode_frame (outbuf, CISsize, tmpbuf, (first == 0) ? 1 : 0, userno);
	if (xwa.xdepth == 16)
	  Color16DitherImage(tmpbuf, vimage.data);
	else if (xwa.xdepth == 32)
	  Color32DitherImage(tmpbuf, vimage.data);

	//	ConvertYUVtoRGB(y1, u1, v1, vimage.data, VDO_WIDTH, VDO_HEIGHT);
	//	memcpy (vimage.data, inbuf, VDO_WIDTH * VDO_HEIGHT * 3/2 );

	/* Display compressed local images */
	BV_display_vdoframe (&xwa, &vimage, &v4l2);

	first ++;  
      }
      else if (param->codec == H263) {
	printf ("Not available.\n");
	exit (0);
      }
      else if (param->codec == JPEG) {
	printf ("Not available.\n");
	exit (0);
      }
      else if (param->codec == MPEG) {
	printf ("Not available.\n");
	exit (0);
      }
      else if (param->codec == RAW) {	/* transmit raw video data */
	BV_packetize_vdoframe (capimg, v4l2.fmt.sizeimage, 
			       VDO_WIDTH, VDO_HEIGHT, tstate, param);

	/* display local video image */
	memcpy (vimage.data, capimg, v4l2.fmt.sizeimage);
	BV_display_vdoframe (&xwa, &vimage, &v4l2);
      }
      else {
	printf ("Error on sending. Specifiy a correct codec type.\n");
	exit (0);
      }
    }

    /* Store compressed video data as a file */
    if (writable)	
      fwrite(vimage.data, 1, CISsize, fp);

    //    thrd_stop (&AR);   /* Thread stop */
  }
}
