
/* *************************************************************************
 *  File : MTvodo.c
 * -------------------------------------------------------------------------
 *  All rights are reserved by Bo-kyun Na, IntellActive Research Inc.
 * -------------------------------------------------------------------------
 * This example reads an image from /dev/video, transmits compressed video 
 * over the local network, and optionally displays the image in Xwindow.
 * ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"

#define STREAMBUFS 4

int once = 1;
int count = 0;
struct pollfd pollfds[2];	/* polling file descriptors for socket */

extern int verbose;


/* *************************************************************************
 * void *Thread_TxVideo (void *arg)
 *
 *
 * ************************************************************************* */
void main (int argc, char **argv)
{
  FILE *df;
  int first=1, userno = 0;
    
  Parameters *param;
  TransceiveState *tstate;

  int vid;
  int xdepth = 0;
  int CISsize;

  V4L2 v4l2;
  GC gc_send_img;
  Widget viewport;
  XtAppContext	app;
  char	my_device[64];
  VIMAGE vimage[STREAMBUFS];

  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 
  char outbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  unsigned char *y, *u, *v;


  /* Initialization of video devices */
  param = (Parameters *) init_TxVidParam(argc, argv);

  param->data_via_file = 0;

  /* Pointers for the conversion of RGBtoYUV */
  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  /* Put in the device node name */
  strcpy(my_device, VDO_DEVICE);

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, VIDEO);

  /* Function: Init. of video capture to get the video from my camera */
  vid = BV_init_vidcap (my_device, &v4l2);

  v4l2.req.count = 2;		/* Num of requested window displays */

  gc_send_img =			/* Init. and format of xwindow display */
    BV_init_display (&app, &xdepth, &viewport, &v4l2, argc, argv, vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_encoder ();		/* Initilization of H261 encoder */
  init_decoder ();		/* Initilization of H261 decoder */

  if (param->datafile)
    df = fopen(param->datafile, "w");

  for (;;) {
    //    sem_wait (&VS);
    //    fprintf (stderr, "SendVidThread ");

    BV_get_vdoframe (vid, vimage[0].data, &v4l2);

    //  BV_display_vdoframe (&viewport, &vimage[0], &gc_send_img, &v4l2, &app);

    /* Conversion image pixel format */
    RGB16toYUV(vimage[0].data, y, u, v, VDO_WIDTH, VDO_HEIGHT);

    /* Implemenation of H261 encoder */
    CISsize = p64EncodeIntraFrame (inbuf, outbuf);
    /*	CISsize = p64EncodeFrame (inbuf, outbuf); */

/*    if (first) {
      CISsize = p64EncodeIntraFrame (inbuf, outbuf);
      first = 0;
    } 
    else {
      CISsize = p64EncodeFrame (inbuf, outbuf);
    }
*/
    decode_frame (outbuf, CISsize, vimage[1].data, first, userno);
    BV_display_vdoframe (&viewport, &vimage[1], &gc_send_img, &v4l2, &app);

    //    if (verbose) {
    //      printf ("raw image = %d bytes\n", v4l2.fmt.sizeimage);
      printf ("Tx:encoded message size = %d\n", CISsize);
      //    }
    
    /* Store compressed video data as a file */
    if (param->datafile)	
      fwrite(vimage[0].data, 1, CISsize, df);

    /* Packetization and sending of video data */
    if (param->transmit) {
      /* transmit compressed image sequence data */
      BV_packetize_vdoframe (outbuf, CISsize, VDO_WIDTH, VDO_HEIGHT, tstate, param);

      /* transmit raw video data */
    //      BV_packetize_vdoframe (vimage[0].ximage->data, v4l2.fmt.sizeimage, VDO_WIDTH, VDO_HEIGHT, tstate, param);

    }

    //    thrd_stop (&AR); 
  }
}
