
/* *************************************************************************
 *  File : MTvodo.c
 * -------------------------------------------------------------------------
 *  All rights are reserved by Bo-kyun Na, IntellActive Research Inc.
 * -------------------------------------------------------------------------
 * This example reads an image from /dev/video, transmits compressed video 
 * over the local network, and optionally displays the image in Xwindow.
 * ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>
//#include <sys/audioio.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"
#include "ext-pthread.h"

#define BUFFER_SIZE (1024*4)
#define STREAMBUFS 4
// #define _POSIX_C_SOURCE 199506L

pthread_mutex_t utex_lock = PTHREAD_MUTEX_INITIALIZER;

int count = 0;
sem_t VS, VR, AS, AR;

struct pollfd pollfds[2];	/* polling file descriptors for socket */

extern int verbose;
extern void create_message_h261 (MsgType *msg, PacketType *packetlist[]);
extern int input_fd, output_fd;

void *Thread_TxVideo (void *);
void *Thread_RxVideo (void *);
extern void *Thread_TxAudio (void *);
extern void *Thread_RxAudio (void *);


/* *************************************************************************
 * thrd_stop (sem_t *sem)
 *
 * ************************************************************************* */
void thrd_stop (sem_t *sem)
{
  pthread_mutex_lock (&utex_lock);
  (count) ? sem_post (sem), count = 0 : count++;
  pthread_mutex_unlock (&utex_lock);
}


/* *************************************************************************
 * usage ()
 *
 *
 * ************************************************************************* */
void usage (char *pgm)
{
  fprintf (stderr, "usage: %s [options] \n \
Legal options:                                       Defaults\n \
<-i file_name> : audio input file name               0\n \
<-o file_name> : audio output file name              0\n \
", pgm);
}


/* *************************************************************************
 * void *Thread_TxVideo (void *arg)
 *
 *
 * ************************************************************************* */
void *Thread_TxVideo (void *arg)
{
  FILE *df;

  //  int frames;
  //  int first_frame = 1;
  unsigned int width, height;
    
  Parameters *param = (Parameters *) arg;
  TransceiveState *tstate;

  int vid;
  int xdepth = 0;
  V4L2 v4l2;
  GC gc_send_img;
  Widget viewport;
  XtAppContext	app;
  char	my_device[64];
  VIMAGE vimage[STREAMBUFS];
  int CISsize;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 
  char outbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  unsigned char *y, *u, *v;


  /* Pointers for the conversion of RGBtoYUV */
  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  /* Put in the device node name */
  strcpy(my_device, VDO_DEVICE);

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, VIDEO);

  /* Function: Init. of video capture to get the video from my camera */
  vid = BV_init_vidcap (my_device, &v4l2);

  v4l2.req.count = 1;		/* Num of requested window displays */

  gc_send_img =			/* Init. and format of xwindow display */
    BV_init_display (&app, &xdepth, &viewport, &v4l2, param->argc, param->argv, vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_encoder ();		/* Initilization of H261 encoder */

  if (param->datafile)
    df = fopen(param->datafile, "w");

  for (;;) {
    sem_wait (&VS);
    fprintf (stderr, "SendVidThread ");

    BV_get_vdoframe (vid, vimage[0].data, &v4l2);
    BV_display_vdoframe (&viewport, &vimage[0], &gc_send_img, &v4l2, &app);
    //    RGB16toYUV(vimage[0].data, y, u, v, VDO_WIDTH, VDO_HEIGHT);

    /* Implemenation of H261 encoder */
    CISsize = p64EncodeIntraFrame (inbuf, outbuf);
    /*	CISsize = p64EncodeFrame (inbuf, outbuf); */

    if (param->datafile)	/* Store compressed video data as a file */
      fwrite(vimage[0].data, 1, CISsize, df);

    if (param->transmit)     /* Packetization and sending of video data */
      //      BV_packetize_vdoframe (outbuf, CISsize, width, height, tstate, param);

      BV_packetize_vdoframe (vimage[0].ximage->data, sizeof (vimage[0].ximage->data), width, height, tstate, param);    
    thrd_stop (&AR); 
    /*   thrd_stop (&AS); */
  }
}


/* ********************************************************************
 * Thread : void *Thread_RxVideo(void *arg)
 *
 *
 * ******************************************************************** */
void *Thread_RxVideo (void *arg)
{
  //  int vid;
  int xdepth = 0;
  int CISsize, first=1, userno = 0;
  uchar *y, *u, *v;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 
  char outbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  V4L2 v4l2;
  VIMAGE vimage[STREAMBUFS];
  Widget viewport;
  XtAppContext	app;
  GC gc_recvd_img;

  int packetcnt = 0;
  int currentframe;
  int lastw, lasth;
  int i;
  unsigned int currentsession;
  unsigned short myflags;

  Parameters *params = (Parameters *) arg;
  TransceiveState *receiver;
  PacketType packets[MAXPACKETS];	/* circular buffer of rec'vd packets */
  PacketType *packetlist[MAXPACKETS];	/* packets for this frame */
  PacketType *curpacket;
  //  MsgType message;


  /* .... initialize everything ....  */
  receiver = initialize_receive(params, VIDEO);

  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  receiver->session = 1;	/* choose remote host to receive */
  v4l2.req.count = 1;		/* Num of requested window displays */

  /* 
  (char *) receiver->name[receiver->session].sin_addr = 
                                  (char *) params->hostname[receiver->session];
  printf ("remotehost = %s\n", receiver->name[receiver->session]);
  */

  /* Initialization of image display  */
  gc_recvd_img = BV_init_display (&app, &xdepth, &viewport, &v4l2, 
				  params->argc, params->argv, vimage);

  /* Initilization of H261 decoder */
  init_codec ();		/* Initialization of codec of H261 */
  init_decoder ();	

  /* .... receive video ....      */
  currentframe = 0;
  currentsession = 0;
  lastw = params->width;
  lasth = params->height;
   
  myflags = 0xffff;
  for (i = 0; i < MAXPACKETS; i++) packetlist[i] = NULL;

  for (;;) {
    sem_wait (&VR);
    sem_post (&VS); 

    fprintf (stderr, "RecvVidThread ");

    curpacket = &packets[packetcnt++];
    if (packetcnt >= MAXPACKETS) packetcnt = 0;
printf ("Rx:1\n");
    /* Get video packet from remote if available. */
    CISsize = receive_packet(receiver, curpacket);

    if (currentsession != curpacket->header.session) {
      /* if you join a conference in progress then the current frame and 
       * session will be grabbed from the first received packet received.  */
      if (verbose) {
	fprintf(stderr, "changing session from %d to %d\n",
		currentsession, curpacket->header.session);
      }
      for (i = 0; i < MAXPACKETS; i++) 	packetlist[i] = NULL;
      myflags = 0xffff;
      currentframe = curpacket->header.frame;
      currentsession = curpacket->header.session;
    }

    /* Resize of window :
     * if this is the first packet of a frame, check for a resize.
     * In BuenaVista, it should be removed. (x and y in packe header).
     */
    if ((curpacket->header.flags & 1) == 0) {
      /* the size of each frame is sent on the first packet of each frame */
      if ((curpacket->header.x != lastw) || (curpacket->header.y != lasth)) {
	lastw = curpacket->header.x;
	lasth = curpacket->header.y;

	if (verbose) 
	  printf("resizing video %dx%d\n",lastw, lasth);

	/* Resize of window */
	/*	resize_window (window, lastw, lasth); */
      }
    }

    if (curpacket->header.frame < currentframe) {
      /* if this packet is from an earlier frame, disregard it. */
      if (verbose)
	fprintf(stderr, "received stale packet %d, current frame %d\n",
		curpacket->header.frame, currentframe);
      continue;
    }

    if (curpacket->header.frame > currentframe) {
      /* If the frame number increased then we probably missed the last
       * packet of the previous frame. We display what we can drop of the
       * previous frame if '-b' option for verbose was selected, otherwise 
       * we just drop the partial frame.                                  
       */
      if (myflags != 0) {
	if (verbose)
	  fprintf(stderr, "dropping %d due to getting packet %d (flags %x)\n",
		  currentframe, curpacket->header.frame, myflags);
      }

      for (i = 0; i < MAXPACKETS; i++) packetlist[i] = NULL;
      myflags = 0xffff;
      currentframe = curpacket->header.frame;
    }

    /* Save the packet for decoding */
    packetlist[packetnum(curpacket->header.flags)] = curpacket;
    myflags &= curpacket->header.flags;

    //    CISsize = ;
    //    decode_frame (inbuf, CISsize, curpacket->payload, first, userno);
    //    YUVtoRGB(vimage[1].data, y, u, v, VDO_WIDTH, VDO_HEIGHT);

    if (myflags == 0) {
      //      if (params->do_cis == H261) {
      //	create_message_h261 (&message, packetlist);
      //      }
      //      else 
      //	create_message(&message, packetlist);

      //      BV_display_vdoframe (&viewport, &vimage[1], &gc_recvd_img, &v4l2, &app);
      //      BV_display_vdoframe (&viewport, inbuf, &gc_recvd_img, &v4l2, &app);
      /* Display raw image data */
      vimage[0].ximage->data = curpacket->payload; 
      BV_display_vdoframe (&viewport, &vimage[0], &gc_recvd_img, &v4l2, &app);
    }

    thrd_stop (&AR); 
    /*    thrd_stop (&AS); */
  }
}


/* ********************************************************************
 * Thread : void *Thread_TxAudio(void *arg)
 *
 *
 * ******************************************************************** */
void *Thread_TxAudio (void *arg)
{
  TransceiveState *TxState;
  Parameters *param;
  
  param = (Parameters *) arg;

  TxState = (TransceiveState *) initialize_transmit (param, AUDIO);

  for ( ; ; ) {
    sem_wait (&AS);

    fprintf (stderr, "SendAudThread\n");



    thrd_stop (&VR);  
  }


}


/* ********************************************************************
 * Thread : void *Thread_RxAudio(void *arg)
 *
 *
 * ******************************************************************** */
void *Thread_RxAudio (void *arg)
{

  Parameters *param = (Parameters *) arg;
  TransceiveState *RxState;

  RxState = (TransceiveState *) initialize_receive (param, AUDIO);

  for ( ; ; ) {
    sem_wait (&AR);
    sem_post (&AS);

    fprintf (stderr, "RecvAudThread ");



    thrd_stop (&VR);
  }

}


/* ***************************************************************
 * Function: void main (int argc, char **argv) 
 *
 *
 * *************************************************************** */
void main (int argc, char **argv)
{
  int   i;
  char aud_fname[80];

  Parameters *Tparam,
             *Rparam;

  pthread_t tid_VidTx,
            tid_VidRx,
            tid_AudRx, 
            tid_AudTx;

  pthread_attr_t attr;

  SEM_INIT (&VS, 0, 0);
  SEM_INIT (&VR, 0, 0);
  SEM_INIT (&AS, 0, 0);
  SEM_INIT (&AR, 0, 0);

  PTHREAD_ATTR_INIT (&attr);
  pthread_attr_setscope (&attr, PTHREAD_SCOPE_SYSTEM);


  for (i=1; i<argc; i++){
    switch (argv[i][1])
      {
      case 'i':
	//	strcpy(aud_fname, argv[++i]);
	//	if (audio_isaudiofile(aud_fname)) {
	//	  input_fd = open(aud_fname, O_RDONLY, 0666);
	//	  if (input_fd < 0)
	//	    fprintf(stderr,"could not open %s\n", aud_fname);
	//	} 
	//	else {
	//	  fprintf(stderr,"file %s is not an audio file\n", aud_fname);
	//	}
	break;
      case 'o':
	//	strcpy(aud_fname, argv[++i]);
	//	output_fd = open(aud_fname, O_WRONLY | O_CREAT | O_TRUNC, 0666);
	//	if (output_fd < 0)
	//	  fprintf(stderr,"could not create %s\n", aud_fname);
	break;
      case '?':
      default:
        usage(argv[0]);
        exit(0);
      }
  }

  /* Initialization of video devices */
  Tparam = (Parameters *) init_TxVidParam(argc, argv);
  Rparam = (Parameters *) init_RxVidParam(argc, argv);
  
  Tparam->data_via_file = Rparam->data_via_file = 0;
  Rparam->argc = argc;
  Rparam->argv = argv;

  sem_post (&VR);

  /* Creation of send/receive video threads */
  sleep (1);
  PTHREAD_CREATE (&tid_VidTx, &attr, Thread_TxVideo, (void *) Tparam);

  sleep (1);
  PTHREAD_CREATE (&tid_VidRx, &attr, Thread_RxVideo, (void *) Rparam);
printf ("main:1:OK\n");
  /* Initialization of audio devices */
  //  if (BV_init_audio ()) {
  //    fprintf (stderr, "Error: BV_init_audio\n");
  //    exit (0);
  //  }

  sleep (1);
  PTHREAD_CREATE (&tid_AudRx, &attr, Thread_RxAudio, (void *) Rparam);
      
  sleep (1);
  PTHREAD_CREATE (&tid_AudTx, &attr, Thread_TxAudio, (void *) Tparam);

  pthread_exit ((void *) NULL);
}
