
/* *************************************************************************
 *  File : netmedia.c
 * -------------------------------------------------------------------------
 *  All rights are reserved by Bo-kyun Na, IntellActive Research Inc.
 * -------------------------------------------------------------------------
 * This example reads an image from /dev/video, transmits compressed video 
 * over the local network, and optionally displays the image in Xwindow.
 * ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "ext-pthread.h"
#include "vdo.h"		/* video related header */
#include "h261.h"
#include "odo.h"		/* audio related headers */

/* Thread related variables */
pthread_mutex_t utex_lock = PTHREAD_MUTEX_INITIALIZER;
int count = 0;			/* used in thrd_stop () */
sem_t VS, VR, AS, AR;


unsigned char *clp;
void error ();

unsigned long wpixel[3];
int expand;

int verbose = 0;

void *Thread_TxVideo (void *);
void *Thread_RxVideo (void *);
void *Thread_TxAudio (void *);
void *Thread_RxAudio (void *);



/* *************************************************************************
 * thrd_stop (sem_t *sem)
 *
 * ************************************************************************* */
void thrd_stop (sem_t *sem)
{
  pthread_mutex_lock (&utex_lock);
  (count) ? sem_post (sem), count = 0 : count++;
  pthread_mutex_unlock (&utex_lock);
}


/* *************************************************************************
 * usage ()
 *
 *
 * ************************************************************************* */
void usage (char *pgm)
{
  fprintf (stderr, "usage: %s [options] \n \
Legal options:                                       Defaults\n \
<-i file_name> : audio input file name               0\n \
<-o file_name> : audio output file name              0\n \
", pgm);

  exit (1);
}


/* *************************************************************************
 * Function    : void *Thread_TxVideo (void *arg)
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
void *Thread_TxVideo (void *arg)
{
  int i;

  Parameters *param = (Parameters *) arg;
  TransceiveState *tstate;

  int vid;
  unsigned char *capimg;	/* buffer for captured image */
  XWA xwa;
  V4L2 v4l2;
  VIMAGE vimage;		/* func. can't pass & return struct,    */
				/* instead,  fun. can do pt. of struct  */	

  int first = 0;
  int CISsize;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 
  char outbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  char tmpbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  unsigned char *y, *u, *v;
  unsigned char *y1, *u1, *v1;
  int userno = 0;


  /* Pointers for the conversion of RGBtoYUV */
  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  y1 = tmpbuf;
  u1 = &tmpbuf[VDO_WIDTH * VDO_HEIGHT];
  v1 = &tmpbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, VIDEO);

  /* Init. of xwindow depth */
  xwa.xdepth = param->depth; 
  if (xwa.xdepth == 15)
    v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB555;
  else if (xwa.xdepth == 16)
    v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB565;
  else if (xwa.xdepth == 24)
    v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR24;
  else if (xwa.xdepth == 32)
    v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR32;
  else {
    printf("Unrecognized display depth. Type correct depth\n");
    usage(param->argv[0]);
    exit (0);
  }

  /* Function: Init. of video capture to get the video from my camera */
  vid = BV_init_vidcap (param->dev_vdo, &v4l2);

  /* Init. and format of xwindow display */
  vimage.data = BV_init_display(&xwa, &v4l2, param->argc, param->argv, &vimage);
  if (vimage.data == NULL) {
    printf("malloc(%d) for vimage.data failed\n", v4l2.fmt.fmt.pix.sizeimage);
    exit (1);
  }

  capimg = (unsigned char *) malloc (v4l2.fmt.fmt.pix.sizeimage);
  if (capimg == NULL) {
    printf("malloc(%d) for capimg failed\n", v4l2.fmt.fmt.pix.sizeimage);
    exit (1);
  }

  /* Initialization of codec functions */
  init_codec ();		/* Initialization of codec of H261 */
  init_encoder ();		/* Initilization of H261 encoder */
  init_decoder ();

  /* clip table */
  if (!(clp = (unsigned char *) malloc(1024))) error("malloc failed\n");
  clp += 384;
  for (i=-384; i<640; i++) clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

  /* Loop processing related to a video frame */
  for (;;) {
    //    sem_wait (&VS);
    //    sem_post (&VR);
    if (verbose) fprintf (stderr, "SendVidThread ");

    BV_get_vdoframe (vid, capimg, &v4l2);

    /* Packetization and sending of video data */
    if (param->transmit) {
      if (param->codec == H261) {
	/* Conversion image pixel format */
	RGB16toYUV(capimg, y, u, v, VDO_WIDTH, VDO_HEIGHT);

	CISsize = ((first %= 10) == 0) ? 
	          p64EncodeIntraFrame (inbuf, outbuf):
	          p64EncodeFrame (inbuf, outbuf);
 
	/* transmit compressed image sequence data */
	BV_packetize_vdoframe(outbuf,CISsize,VDO_WIDTH,VDO_HEIGHT,tstate,param);

	/* Decode of recvd message and conversion to RGB from YUV  */
	decode_frame (outbuf, CISsize, tmpbuf, (first == 0) ? 1 : 0, userno);

	ConvertYUVtoRGB(y1, u1, v1, vimage.data, VDO_WIDTH, VDO_HEIGHT);

	/* Display compressed local images */
	BV_display_vdoframe (&xwa, &vimage, &v4l2);

	first ++;  
      }
      else if (param->codec == H263) {
	printf ("Not available.\n");
	exit (0);
      }
      else if (param->codec == JPEG) {
	printf ("Not available.\n");
	exit (0);
      }
      else if (param->codec == MPEG) {
	printf ("Not available.\n");
	exit (0);
      }
      else if (param->codec == RAW) {	/* transmit raw video data */
	BV_packetize_vdoframe (capimg, v4l2.fmt.fmt.pix.sizeimage, 
			       VDO_WIDTH, VDO_HEIGHT, tstate, param);

	/* display local video image */
	memcpy (vimage.data, capimg, v4l2.fmt.fmt.pix.sizeimage);
	BV_display_vdoframe (&xwa, &vimage, &v4l2);
      }
      else {
	printf ("Error on sending. Specifiy a correct codec type.\n");
	exit (0);
      }
    }

    /* Store compressed video data as a file */
    //    if (param->wrfile != NULL)	
    //      fwrite(vimage.data, 1, CISsize, fp);
    //    sem_post (&VR);
    //    thrd_stop (&AS);   /* Thread stop */
  }
}


/* *************************************************************************
 * Function    : void *Thread_RxVideo (void *arg)
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
void *Thread_RxVideo (void *arg)
{
  int i;
  int userno = 0;
  unsigned char *y, *u, *v;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 

  V4L2 v4l2;
  XWA xwa;
  VIMAGE vimage;

  Parameters *param = (Parameters *) arg;
  TransceiveState *receiver;
  MsgType *message;


  /* set pointers of codec buffer */
  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  /* Initialization of network environments */
  receiver = initialize_receive(param, VIDEO);

  message = (MsgType *) malloc (sizeof (MsgType));
  if (message == NULL) {
    printf ("malloc fail: message = %d\n", sizeof (MsgType));
    exit (1);
  }

  /* Init. of xwindow depth */
  xwa.xdepth = param->depth; 
  if (xwa.xdepth == 15)
    v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB555;
  else if (xwa.xdepth == 16)
    v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB565;
  else if (xwa.xdepth == 24)
    v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR24;
  else if (xwa.xdepth == 32)
    v4l2.fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR32;
  else {
    printf("Unrecognized display depth. Type correct depth\n");
    usage(param->argv[0]);
    exit (0);
  }

  /* Initialization of image display  */
  vimage.data = BV_init_display(&xwa, &v4l2, param->argc, param->argv, &vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_decoder ();		/* Initilization of H261 decoder */

  /* clip table */
  if (!(clp = (unsigned char *) malloc(1024))) error("malloc failed\n");
  clp += 384;
  for (i=-384; i<640; i++) clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

  if (param->wrfile != NULL) param->fp = fopen(param->wrfile, "w");

  for (;;) {
    //    sem_wait (&VR);
    if (verbose) fprintf (stderr, "RecvVidThread ");

    message = (MsgType *) BV_VDO_receive (receiver, message);

    if (param->codec == H261) {
      /* Decode of recvd message and conversion to RGB from YUV  */
	decode_frame (message->data, message->header.size, inbuf, 
		      ((message->header.frame % 10) == 0) ? 1 : 0, userno);
	ConvertYUVtoRGB(y, u, v, vimage.data, VDO_WIDTH, VDO_HEIGHT);
	//	memcpy(vimage.data, message->data, message->header.size); 
    } 
    else if (param->codec == H263) {
      printf ("Not yet served this codec\n");
      exit (0);
    }
    else if (param->codec == JPEG) {
      printf ("Not yet served this codec\n");
      exit (0);
    }
    else if (param->codec == MPEG) {
      printf ("Not yet served this codec\n");
      exit (0);
    }
    else if (param->codec == RAW) {
      memcpy(vimage.data, message->data, message->header.size); 
    }
    else {
      printf ("Error on compression type: Specify the codec type\n");
      exit (0);
    }

    if (verbose)
      printf ("Rx: The %dth image frame w/ size = %d bytes\n", 
	      message->header.frame, message->header.size);
    
    /* Display images */
    BV_display_vdoframe (&xwa, &vimage, &v4l2);

    /* Store received video data as a file */
    //      if (param->wrfile)	
    //	fwrite(vimage.data, 1, message.header.size, df);

  }

  //  thrd_stop (&AS); 
}



/* *************************************************************************
 * Function    : void *Thread_TxAudio (void *arg)
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
void *Thread_TxAudio (void *arg)
{
  int fd;			/* sound device file descriptor */
  //  int status;			/* return status of system calls */
  int rd;			/* size in byte unit */
  int odo_frame;

  unsigned char *buf;		/* this buffer holds the digitized audio */

  /* Network parameters */
  Parameters *param = (Parameters *) arg;
  TransceiveState *tstate;


  odo_frame = ODO_LENGTH * ODO_RATE * ODO_SIZE * ODO_CHANNELS / 8;
  buf = (unsigned char *) malloc(odo_frame);
  if (buf == NULL) {
    printf ("malloc fail: sizeof (buf) = %d\n", odo_frame);
    exit (1);
  }

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, AUDIO);

  for ( ; ; ) {			/* loop until Control-C */
    sem_wait (&AS);		
    if (verbose) fprintf (stderr, "SendAudThread\n");

    /* Initialization of audio */
    fd = BV_ODO_initialize (param->dev_odo);
    rd = BV_ODO_record (fd, buf, odo_frame);
    if (verbose) printf ("Recorded audio size is %d\n", rd);
    close (fd);

    BV_odo_packetize (buf, odo_frame, tstate, param);

    /* echo recorded audio */
    //    BV_ODO_play (fd, buf, odo_frame);
    /* wait for playback to complete before recording again */
    //    status = ioctl(fd, SOUND_PCM_SYNC, 0); 
    //    if (status == -1)
    //      perror("SOUND_PCM_SYNC ioctl failed");

    if (verbose) printf ("AudioSend Thread done\n");
    sem_post (&AR);		/* only for audio */
    //    thrd_stop (&VS);
  }
}



/* *************************************************************************
 * Function    : Thread : void *Thread_RxAudio(void *arg)
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
void *Thread_RxAudio (void *arg)
{
  int fd;			/* sound device file descriptor */
  int wr;			/* played audio size in byte unit */
  int status;			/* return status of system calls */
  int odo_frame;		/* size of audio buffer */

  unsigned char *buf;		/* this buffer holds the digitized audio */

  Parameters *param = (Parameters *) arg;
  TransceiveState *RxState;
  MsgType *message;


  odo_frame = ODO_LENGTH * ODO_RATE * ODO_SIZE * ODO_CHANNELS / 8;
  buf = (unsigned char *) malloc(odo_frame);
  if (buf == NULL) {
    printf ("malloc fail: sizeof (buf) = %d\n", odo_frame);
    exit (1);
  }

  message = (MsgType *) malloc (sizeof (MsgType));
  if (message == NULL) {
    printf ("malloc fail: message = %d\n", sizeof (MsgType));
    exit (1);
  }

  /* Init. of network */
  RxState = (TransceiveState *) initialize_receive (param, AUDIO);

  /* loop for audio play until Control-C */
  for ( ; ; ) {		
    sem_wait (&AR);
    if (verbose) 
      fprintf (stderr, "RecvAudThread ");

    message = (MsgType *) BV_ODO_receive (RxState, message);

    /* Initialization of audio */
    fd = BV_ODO_initialize (param->dev_odo);

    /* play audio data */
    wr = BV_ODO_play (fd, message->data, odo_frame);
    if (verbose) printf ("Played audio size is %d bytes\n", wr);

    /* wait for playback to complete before recording again */
    status = ioctl(fd, SOUND_PCM_SYNC, 0); 
    if (status == -1)
      perror("SOUND_PCM_SYNC ioctl failed");
    close (fd);

    if (verbose) printf ("AudioReceive Thread done\n");
    sem_post (&AS);
    //    thrd_stop (&VS);
  }
}


/* *************************************************************************
 * Function    : void main (int argc, char **argv) 
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
int main (int argc, char **argv)
{
  int   i;

  Parameters *param;

  pthread_t tid_VidTx,
            tid_VidRx,
            tid_AudRx, 
            tid_AudTx;

  pthread_attr_t attr;

  SEM_INIT (&VS, 0, 0);
  SEM_INIT (&VR, 0, 0);
  SEM_INIT (&AS, 0, 0);
  SEM_INIT (&AR, 0, 0);

  PTHREAD_ATTR_INIT (&attr);
  pthread_attr_setscope (&attr, PTHREAD_SCOPE_SYSTEM);


  /* Initialization of video and audio parameters  */
  param = (Parameters *) init_Param(argc, argv);
  param->argc = argc;
  param->argv = argv;

  /* audio device node name */
  strcpy (param->dev_odo, ODO_DEVICE);		

  /* video device node name */
  strcpy(param->dev_vdo, VDO_DEVICE);

  /* Process command line arguments */
  for (i = 1; i < argc; i++) {
    if (!strcmp (argv[i], "-C")) {
      if (!strcmp(argv[++i], "H261")) {
      	param->codec = H261;
      	param->cis_type = "H261";
      }
      else if (!strcmp(argv[i], "H263")) {
      	param->codec = H263;
      	param->cis_type = "H263";
      }
      else if (!strcmp(argv[i], "JPEG")) {
      	param->codec = JPEG;
      	param->cis_type = "Jpeg";
      }
      else if (!strcmp(argv[i], "MPEG")) {
      	param->codec = MPEG;
      	param->cis_type = "Mpeg1";
      }
      else if (!strcmp(argv[i], "RAW")) {
	param->codec = RAW;
      	param->cis_type = "RAW";
      }
      else {
	printf ("Specifiy your codec mode\n");
        usage(argv[0]);
      }
      if (verbose)
	printf ("codec is %s\n", param->cis_type);
    }
    if(!strcmp(argv[i], "-D")) {
      strcpy (param->dev_vdo, argv[++i]);
    }
    if(!strcmp(argv[i], "-d")) {
      param->depth = atoi(argv[++i]);
      if (param->depth != (15 || 16 || 24 || 32)) {
	printf("Unrecognized display depth. Type correct depth\n");
        usage(argv[0]);
      }
    }
    if (!strcmp(argv[i], "-i")) {
      strcpy (param->rdfile, argv[++i]);
      param->fp = fopen(param->rdfile, "r");
      if (param->fp == NULL) 
	printf ("Can't open image file: %s\n", param->rdfile);
      else  printf ("Image file, \"%s\", is opened.\n", param->rdfile);
    }
    if (!strcmp(argv[i], "-o")) {
      strcpy (param->wrfile, argv[++i]);
      param->fp = fopen(param->wrfile, "w");
      if (param->fp == NULL) 
	printf ("Can't open image file: %s\n", param->wrfile);
      else  printf ("Image file, \"%s\", is opened\n", param->wrfile);
    }
    //    if(!strcmp(argv[i], "-r")) {
    //    rate = atoi(argv[++i]);
    //    }
    if(!strcmp(argv[i], "-s")) {
      param->width = atoi(argv[++i]);
      param->height = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-v")) {
      verbose = 1;
    }
  }

  //  sem_post (&VS);		/* If AR everybody wait to recv. So, AS! */
  sem_post (&AS);
  /* Creation of send/receive video threads */
  sleep (1);
  PTHREAD_CREATE (&tid_VidTx, &attr, Thread_TxVideo, (void *) param);

  sleep (1);
  PTHREAD_CREATE (&tid_VidRx, &attr, Thread_RxVideo, (void *) param);

  sleep (1);
  PTHREAD_CREATE (&tid_AudTx, &attr, Thread_TxAudio, (void *) param);      

  sleep (1);
  PTHREAD_CREATE (&tid_AudRx, &attr, Thread_RxAudio, (void *) param);  

  pthread_exit ((void *) NULL);
}

/* *************************************************************************
 * Function    : 
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
