/* @(#)MTsend.c	v0.0 1999/02/03  */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>		/* only for linux */
#include <malloc.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>		/* only for linux */

#include "network.h"
#include "params.h"
#include "transceive.h"


/* *************************************************************************
 * TransceiveState *initialize_transmit(Parameters *param, int media_type)
 *
 *
 * ************************************************************************* */
TransceiveState *initialize_transmit(Parameters *param, int media_type)
{
  TransceiveState *transmitter;
  int Txsock;
  int on = 1;
  char hostname[MAXHOSTNAMELEN + 1];
  struct hostent *host, *gethostbyname();

  /* .... create transmitter state ....  */
  transmitter = (TransceiveState *) calloc(1, sizeof(TransceiveState));
  if (transmitter == NULL)
    return (NULL);

  /* .... open transmitter socket and set broadcast option ....   */
  if ((Txsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("Transmit socket");
    free(transmitter);
    return (NULL);
  }

  if (setsockopt(Txsock, SOL_SOCKET, SO_BROADCAST, (char *) &on, sizeof(on))) {
    perror("Transmit setsockopt");
    close(Txsock);
    free(transmitter);
    return (NULL);
  }
  transmitter->socket = Txsock;

  /*  If there is no specific transmission destination, specified by a  *
   *  host.domain string, then broadcast on the local network           */
  if (param->hostnameknt == 0) {
    /* .... use local hostname to identify local network .... */
    if (gethostname(hostname, sizeof(hostname)) < 0) {
      perror("Transmit gethostname");
      close(Txsock);
      free(transmitter);
      return (NULL);
    }
    if (verbose)
      fprintf(stderr, "Broadcasting on local net from %s\n", hostname);

    if ((host = gethostbyname(hostname)) == NULL) {
      perror("Transmit gethostbyname");
      close(Txsock);
      free(transmitter);
      return (NULL);
    }

    memcpy((char *) &transmitter->name[0].sin_addr, (char *) host->h_addr, host->h_length);

    transmitter->name[0].sin_addr.s_addr &= IN_CLASSC_NET;
    transmitter->name[0].sin_family = AF_INET;
    transmitter->name[0].sin_port = CHANNEL_BASE + param->channel + media_type;
    transmitter->nameknt = 1;
  }
  else {
    int i;
    int subflag;
    char *tptr;

    for (i = 0; i < param->hostnameknt; i++) {
      tptr = param->hostname[i];
      subflag = 0xffffffff;
      if (tptr[0] == 'S') {
	subflag = 0xffffff00;
	tptr++;
      }

      if (tptr[0] >= '0' && tptr[0] <= '9') {
	transmitter->name[transmitter->nameknt].sin_addr.s_addr = inet_addr(tptr);
	transmitter->name[transmitter->nameknt].sin_addr.s_addr &= subflag;
	transmitter->name[transmitter->nameknt].sin_family = AF_INET;
	transmitter->name[transmitter->nameknt].sin_port = CHANNEL_BASE + 
	  param->channel + media_type;
	transmitter->nameknt++;
      }
      else {
	if ((host = gethostbyname(tptr)) != NULL) {
	  memcpy((char *) &transmitter->name[transmitter->nameknt].sin_addr,
		 (char *) host->h_addr, host->h_length);
	  transmitter->name[transmitter->nameknt].sin_addr.s_addr &=  subflag;
	  transmitter->name[transmitter->nameknt].sin_family =	AF_INET;
	  transmitter->name[transmitter->nameknt].sin_port = 
	    CHANNEL_BASE + param->channel + media_type;
	  transmitter->nameknt++;
	}
	else {
	  fprintf(stderr, "Transmit gethostbyname '%s'  ", tptr);
	  perror("");
	}
      }
    }
  }

  transmitter->session = (unsigned int) getpid();
  if (verbose) {
    char *tname;
    int i;
    
    if (verbose)
      fprintf(stderr, "Session number = %d\n", transmitter->session);

    for (i = 0; i < transmitter->nameknt; i++) {
      tname = "SUBNET";
      if (transmitter->name[i].sin_addr.s_addr & 0xff) {
	if ((host = gethostbyaddr((char *) &transmitter->name[i].sin_addr, 4, 
				  AF_INET)) == NULL) {
	  perror("gethostbyaddr");
	  tname = "CAN NOT FIND";
	}
	else {
	  tname = host->h_name;
	}
      }
      
      if (verbose)
	fprintf(stderr, "Sending to %d.%d.%d.%d\t\t%s\n",
		(transmitter->name[i].sin_addr.s_addr >> 24) & 0xff,
		(transmitter->name[i].sin_addr.s_addr >> 16) & 0xff,
		(transmitter->name[i].sin_addr.s_addr >> 8) & 0xff,
		(transmitter->name[i].sin_addr.s_addr >> 0) & 0xff, tname);
    }
  }

  return (transmitter);
}


/* *************************************************************************
 * int transmit_packet(TransceiveState *transmitter, PacketType *packet)
 *
 *
 * ************************************************************************* */
int transmit_packet(TransceiveState *transmitter, PacketType *packet)
{
  int i;
  int len;
  int size;

  size = sizeof(PacketHeader) + PACKETDATASIZE;	/* packet->header.size; */
  for (i = 0; i < transmitter->nameknt; i++) {
    len = sendto(transmitter->socket, (char *) packet, size, 0,
		 (struct sockaddr *) & transmitter->name[i], 
		 sizeof(struct sockaddr_in));
    if (len != size)
      return 0;
  }
  return 1;
}


/* *************************************************************************
 * void close_transmit(TransceiveState * transmitter)
 *
 *
 * ************************************************************************* */
void close_transmit(TransceiveState * transmitter)
{
  close(transmitter->socket);
  free(transmitter);
}


/* *************************************************************************
 * void BV_packetizevdoframe ()
 * :: Packetization of video data
 *
 * ************************************************************************* */
void BV_packetize_vdoframe (uchar *data, int nbytes, int width, int height, 
			    TransceiveState *tstate, Parameters *param)
{
  PacketType packet;
  static int frame = 0;
  int count = 0;
  
  packet.header.frame = frame++;
  packet.header.session = tstate->session | (VIDEO_PACKET << 28);
  packet.header.x = width;
  packet.header.y = height;

  //  printf("transmitting: %d bytes\n", nbytes);

  if (nbytes > PACKETDATASIZE * MAXPACKETS) {
    if (verbose)
      fprintf(stderr, "Not sending frame %d due to size (%d) > maximum\n",
              frame, nbytes);
  }
  else {
    int x = 0;
    int y = 0;
    
    while (nbytes > 0) {
      int curbytes = 0;
      
      if (nbytes <= PACKETDATASIZE) {
        curbytes = nbytes;
	packet.header.flags = ((1 << count) - 1);
        if (count) {
          packet.header.x = x * 4;
          packet.header.y = y * 4;
        }
      }
      else {
        curbytes = PACKETDATASIZE;
        packet.header.flags = ~(1 << count);
      }
      
      packet.header.size = curbytes;
      count++;
      nbytes -= curbytes;
      memcpy (packet.payload, data, curbytes);
      data += curbytes;
      
      if (verbose)
        printf("transmitting: %d bytes left: %d\n",packet.header.size, nbytes);
      
      transmit_packet(tstate, &packet);
      if (param->polling)
        poll(0, 0, param->polling);
    }
  }
}
