/* *************************************************************************
 * Name : vdo.h
 * by   : Bo-kyun Na, IntellActive Research.
 *        bkna@npac.syr.edu
 * Date : 1999.06.05
 * ************************************************************************* */
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>

/* These are needed to use the Videum driver */
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/videodev.h>  /* Video for Linux Two */

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Porthole.h>

/*  Set these according to video capture device set-up */
#define VDO_DEVICE "/dev/video0"
#define VDO_STD	  "NTSC"	/*-> "PAL" or "NTSC" */
#define VDO_WIDTH  176
#define VDO_HEIGHT 144
#define VDO_INPUT  0		/* 0 for MXC, 1 for VID, and 2 for SVHS */
#define VDO_FRAMERATE 1000000	/* 10 million / frames-per-second */

// #define X_DEPTH 16
//#define PIXELFORMAT  V4L2_PIX_FMT_BGR32
// V4L2_PIX_FMT_YUYV
// V4L2_PIX_FMT_GREY
// V4L2_PIX_FMT_YUV420
// V4L2_PIX_FMT_RGB555
// V4L2_PIX_FMT_RGB565
// V4L2_PIX_FMT_BGR24
// V4L2_PIX_FMT_BGR32


typedef struct {
  struct v4l2_buffer vidbuf;
  char	  *data;
  XImage  *ximage;
} VIMAGE;

typedef struct {
  struct v4l2_capability      cap;
  struct v4l2_format	      fmt;
  struct v4l2_standard	      std;
  struct v4l2_streamparm      parm;
} V4L2;

typedef struct {	   /* This struct works with Xtool and Motif libs */
  int xdepth;
  GC gc;
  XtAppContext app;
  Widget viewport;
} XWA;


extern int verbose;

/* ** Prototype of Functions  ** */ 

void check_events(Display *display);

__u32 frame_buffer_format (Display *d, Window w);
void quit_capture (int vid, Widget w, XtPointer client, XtPointer call);
char* 
BV_init_display (XWA *xwa, V4L2 *v4l2, int argc, char **argv, VIMAGE *vimage);
void BV_display_vdoframe (XWA *xwa, VIMAGE *vimage, V4L2 *v4l2);

int BV_init_vidcap (char *my_device, V4L2 *v4l2);
int BV_close_vid (int fd);
void printctrl (struct v4l2_queryctrl *qc);
int BV_get_vdoframe (int vid, char *outbuf, V4L2 *v4l2);

void RGB16toYUV(unsigned char *input, unsigned char *y, unsigned char *u, 
		unsigned char *v, int width, int height);
void ConvertYUVtoRGB (unsigned char *y, unsigned char *u, unsigned char *v,
		      unsigned char *outbuf, int width, int height);

void InitColorDither (int thirty2, XWA *xwa);
void Color16DitherImage(unsigned char *src, unsigned char *dst);
void Color32DitherImage(unsigned char *src, unsigned char *dst);

/* end of vdo.h */
